<?php

namespace Config;

// Create a new instance of our RouteCollection class.
$routes = Services::routes();

// Load the system's routing file first, so that the app and ENVIRONMENT
// can override as needed.
if (is_file(SYSTEMPATH . 'Config/Routes.php')) {
    require SYSTEMPATH . 'Config/Routes.php';
}

/*
 * --------------------------------------------------------------------
 * Router Setup
 * --------------------------------------------------------------------
 */
$routes->setDefaultNamespace('App\Controllers');
$routes->setDefaultController('Home');
$routes->setDefaultMethod('index');
$routes->setTranslateURIDashes(false);
$routes->set404Override();
// The Auto Routing (Legacy) is very dangerous. It is easy to create vulnerable apps
// where controller filters or CSRF protection are bypassed.
// If you don't want to define all routes, please use the Auto Routing (Improved).
// Set `$autoRoutesImproved` to true in `app/Config/Feature.php` and set the following to true.
//$routes->setAutoRoute(false);

/*
 * --------------------------------------------------------------------
 * Route Definitions
 * --------------------------------------------------------------------
 */

// We get a performance increase by specifying the default
// route since we don't have to scan directories.
$routes->group('/', ['namespace' => 'App\Controllers\Web'], function ($routes) {
    $routes->get('', 'Home::index');
    $routes->get('products', 'Marketplace::index');
    $routes->get('district-filter', 'Marketplace::district_filter');
    $routes->get('market-filter', 'Marketplace::market_filter');
    $routes->post('district_ajax', 'Marketplace::district_ajax');
    $routes->post('seller_district_ajax', 'Vendors::seller_district_ajax');
    $routes->post('service_district_ajax', 'Marketplace::service_district_ajax');
    $routes->post('s_category_ajax', 'Marketplace::category_ajax');
    $routes->match(['get', 'post'], 'user-register', 'Marketplace::user_register');
    $routes->post('get-district-markets', 'Marketplace::get_district_markets');

    // $routes->get('sellers', 'Web\Vendors::vendors');
    $routes->match(["get", "post"], "sellers", "Vendors::vendors");
    $routes->get('vendor-profile', 'Vendors::vendor_profile');
    $routes->get('provider-profile', 'Vendors::provider_profile');
    $routes->get('product-details', 'Marketplace::product_details');
    $routes->get('filter', 'Marketplace::filter');
    $routes->get('markets', 'Marketplace::markets');
    $routes->get('price-watch', 'Marketplace::price_watch');
    $routes->get('product-highlights/district/(:any)', 'Marketplace::product_highlights/$1');
    $routes->get('service-highlights/district/(:any)', 'Marketplace::service_highlights/$1');
    $routes->get('product-highlights/market/(:any)', 'Marketplace::market_highlights/$1');

    // Marketplace
    $routes->get("buyers", "Marketplace::buyer_request");
    $routes->match(["get", "post"], "market/(:any)", "Marketplace::market_products/$1");
    $routes->match(["get", "post"], "services-by-category/(:any)", "Marketplace::services_by_category/$1");
    $routes->match(["get", "post"], "seller-search/district/(:any)", "Marketplace::seller_search/$1");
    $routes->match(["get", "post"], "district/(:any)", "Marketplace::district_markets/$1");
    $routes->match(["get", "post"], "services/districts/(:any)", "Marketplace::service_by_district/$1");
    $routes->match(["get", "post"], "sellers/(:any)", "Vendors::vendor_details/$1");
    $routes->match(["get", "post"], "products/(:any)", "Marketplace::product_details/$1");
    $routes->match(["get", "post"], "send_request/(:any)", "Marketplace::send_request/$1");
    $routes->match(["get", "post"], "custom_request", "Marketplace::custom_request");
    $routes->match(["get", "post"], "service_request", "Marketplace::service_request");
    $routes->match(["get", "post"], "report_vendor", "Marketplace::report_vendor");
    $routes->match(["get", "post"], "review_product/(:any)", "Marketplace::review_product/$1");
    $routes->match(["get", "post"], "review_service/(:any)", "Marketplace::review_service/$1");
    $routes->match(["get", "post"], "place_order/(:any)", "Marketplace::place_order/$1");
    $routes->match(["get", "post"], "place_service_order/(:any)", "Marketplace::place_service_order/$1");
    $routes->match(["get", "post"], "services/(:any)", "Marketplace::service_details/$1");
    $routes->get('product-marketplace', 'Marketplace::product_market');
    $routes->get('service-marketplace', 'Marketplace::service_marketplace');
    $routes->get('test-service-marketplace', 'Marketplace::test_service_marketplace');
    // Filtering
    $routes->post('filter_services', 'Marketplace::filter_services');
    $routes->post('filter_products', 'Marketplace::filter_products');
    // $routes->match(["get", "post"], "filter_market/(:any)", "Web\Marketplace::filter_market/$1");
    $routes->post('filter_market', 'Marketplace::filter_market');
    // Services
    $routes->get('services', 'Marketplace::services');
});

// Forum
$routes->group('forum', ['filter' => 'role:Seller'], function ($routes) {
    $routes->get('/', 'Forum\Home::index');
    $routes->match(["get", "post"], "topics/group/(:any)", "Forum\Home::group_topics/$1");
    $routes->match(["get", "post"], "create_topic", "Forum\Home::create_topic");
    $routes->match(["get", "post"], "discussions/topic/(:any)", "Forum\Home::topic_discussions/$1");
});


$routes->group('user', ['filter' => 'role:Buyer'], function ($routes) {
    $routes->get('/', 'Web\Users::dashboard');
    $routes->get('dashboard', 'Web\Users::dashboard');
    $routes->get('requests', 'Web\Users::requests');
    $routes->get('addresses', 'Web\Users::addresses');
    $routes->get('settings', 'Web\Users::settings');
    $routes->get('order-history', 'Web\Users::order_history');
    $routes->match(["get", "post"], "order_invoice/(:any)", "Web\Users::order_invoice/$1");
    $routes->match(["get", "post"], "service_invoice/(:any)", "Web\Users::service_invoice/$1");
    $routes->match(["get", "post"], "edit_billing_info/(:any)", "Web\Users::edit_billing_info/$1");
    $routes->match(["get", "post"], "edit_account/(:any)", "Web\Users::edit_account/$1");
    $routes->match(["get", "post"], "edit_password/(:any)", "Web\Users::edit_password/$1");
    $routes->match(["get", "post"], "request_details/(:any)", "Web\Users::request_details/$1");
});

$routes->group('admin', ['filter' => 'role:Admin'], function ($routes) {
    // Products & Services
    $routes->get('/', 'Admin\Home::index');
    $routes->get('dashboard', 'Admin\Home::index');
    $routes->get('products', 'Admin\Marketplace::products');
    $routes->get('services', 'Admin\Marketplace::services');
    $routes->get('reviews', 'Admin\Marketplace::reviews');
    $routes->match(["get", "post"], "product_reviews/(:any)", "Admin\Marketplace::product_reviews/$1");
    $routes->match(["get", "post"], "product_invoice/(:any)", "Admin\Marketplace::product_invoice/$1");
    $routes->match(["get", "post"], "service_invoice/(:any)", "Admin\Marketplace::service_invoice/$1");
    $routes->get('product-requests', 'Admin\Requests::index');
    $routes->match(["get", "post"], "request-details/(:any)", "Admin\Requests::request_details/$1");
    $routes->get('product-details/(:any)', 'Admin\Marketplace::product_details/$1');
    $routes->get('service-details/(:any)', 'Admin\Marketplace::service_details/$1');
    // Attributes
    $routes->get('categories', 'Admin\Marketplace::categories');
    $routes->match(["get", "post"], "add-categories", "Admin\Marketplace::add_category");
    $routes->match(["get", "post"], "edit_category/(:num)", "Admin\Marketplace::edit_category/$1");
    $routes->get('delete_category/(:num)', 'Admin\Marketplace::delete_category/$1');
    $routes->get('attributes', 'Admin\Marketplace::attributes');
    $routes->match(["get", "post"], "add-attributes", "Admin\Marketplace::add_attributes");
    $routes->match(["get", "post"], "edit_attribute/(:num)", "Admin\Marketplace::edit_attribute/$1");
    $routes->get('delete_attribute/(:num)', 'Admin\Marketplace::delete_attribute/$1');
    $routes->get('units', 'Admin\Marketplace::units');
    $routes->match(["get", "post"], "add-units", "Admin\Marketplace::add_units");
    $routes->match(["get", "post"], "edit_unit/(:num)", "Admin\Marketplace::edit_unit/$1");
    $routes->get('delete_unit/(:num)', 'Admin\Marketplace::delete_unit/$1');
    $routes->get('service-types', 'Admin\Marketplace::service_types');
    $routes->match(["get", "post"], "add-service-type", "Admin\Marketplace::add_service_type");
    $routes->match(["get", "post"], "edit_service_type/(:num)", "Admin\Marketplace::edit_service_type/$1");
    $routes->get('delete_service_type/(:num)', 'Admin\Marketplace::delete_service_type/$1');
    // Transactions
    $routes->get('transactions', 'Admin\Transactions::transactions');
    $routes->get('completed-transactions', 'Admin\Transactions::completed_transactions');
    $routes->get('pending-transactions', 'Admin\Transactions::pending_transactions');
    $routes->get('cancelled-transactions', 'Admin\Transactions::cancelled_transactions');

    $routes->get('service-orders', 'Admin\Transactions::service_orders');
    $routes->get('completed-service-transactions', 'Admin\Transactions::completed_service_transactions');
    $routes->get('pending-service-transactions', 'Admin\Transactions::pending_service_transactions');
    $routes->get('cancelled-service-transactions', 'Admin\Transactions::cancelled_service_transactions');
    // Service Transactions
    $routes->get('s-transactions', 'Admin\Transactions::s_transactions');
    $routes->get('completed-s-transactions', 'Admin\Transactions::completed_s_transactions');
    $routes->get('pending-s-transactions', 'Admin\Transactions::pending_s_transactions');
    $routes->get('cancelled-s-transactions', 'Admin\Transactions::cancelled_s_transactions');
    // Complaints
    $routes->get('complaints', 'Admin\Complaints::index');
    $routes->match(["get", "post"], "customer_complaint/(:any)", "Admin\Complaints::customer_complaint/$1");
    // Farmers
    $routes->get('farmers', 'Admin\Farmers::index');
    $routes->get('confirmed-farmers', 'Admin\Farmers::confirmed_farmers');
    $routes->get('pending-farmers', 'Admin\Farmers::pending_farmers');
    $routes->get('declined-farmers', 'Admin\Farmers::declined_farmers');
    $routes->get('banned-farmers', 'Admin\Farmers::banned_farmers');
    $routes->get('download-doc/(:any)', 'Admin\Farmers::download_doc/$1');
    $routes->match(["get", "post"], "seller-profile/(:any)", "Admin\Farmers::seller_profile/$1");
    $routes->match(["get", "post"], "seller-password/(:any)", "Admin\Farmers::seller_password/$1");
    // Customers
    $routes->get('customers', 'Admin\Customers::index');
    $routes->match(["get", "post"], "customer-profile/(:any)", "Admin\Customers::customer_profile/$1");
    $routes->match(["get", "post"], "customer-password/(:any)", "Admin\Customers::customer_password/$1");
    // Farmer Groups
    $routes->get('farmer-groups', 'Admin\FarmGroups::index');
    $routes->match(["get", "post"], "add-farm-group", "Admin\FarmGroups::add_farm_group");
    $routes->match(["get", "post"], "edit_farm_group/(:num)", "Admin\FarmGroups::edit_farm_group/$1");
    $routes->get('delete_farm_group/(:num)', 'Admin\FarmGroups::delete_farm_group/$1');
    // Markets
    $routes->get('markets', 'Admin\SoftSettings::markets');
    $routes->match(["get", "post"], "add-markets", "Admin\SoftSettings::add_markets");
    $routes->match(["get", "post"], "edit_market/(:num)", "Admin\SoftSettings::edit_market/$1");
    $routes->get('delete_market/(:num)', 'Admin\SoftSettings::delete_market/$1');
    // Software Settings
    $routes->get('districts', 'Admin\SoftSettings::districts');
    $routes->match(["get", "post"], "add-districts", "Admin\SoftSettings::add_districts");
    $routes->match(["get", "post"], "edit_district/(:num)", "Admin\SoftSettings::edit_district/$1");
    $routes->get('delete_district/(:num)', 'Admin\SoftSettings::delete_district/$1');
    $routes->match(["get", "post"], "email-settings", "Admin\SoftSettings::email_settings");
    $routes->post('update-emailsettings/(:num)', 'Admin\SoftSettings::update_emailsettings/$1');
    $routes->get("organ-settings", "Admin\SoftSettings::organ_settings");
    $routes->match(["get", "post"], 'account-settings', 'Admin\SoftSettings::account_settings');
    $routes->post("update-organsettings/(:num)", "Admin\SoftSettings::update_organsettings/$1");
    // Website Settings
    $routes->get('home_sliders', 'Admin\WebSettings::home_sliders');
    $routes->match(["get", "post"], "edit_slider/(:num)", "Admin\WebSettings::edit_slider/$1");
    // Product Reports
    $routes->get('categories-report', 'Admin\Reports::category_report');
    $routes->get('markets-report', 'Admin\Reports::markets_report');
    $routes->get('districts-report', 'Admin\Reports::districts_report');
    $routes->get('sellers-report', 'Admin\Reports::sellers_report');
    $routes->get('customers-report', 'Admin\Reports::customers_report');
    $routes->get('sales-report', 'Admin\Reports::sales_report');
    // Service Reports
    $routes->get('services/categories-report', 'Admin\Reports::services_category_report');
    $routes->get('services/districts-report', 'Admin\Reports::services_districts_report');
    $routes->get('services/sellers-report', 'Admin\Reports::services_sellers_report');
    $routes->get('services/customers-report', 'Admin\Reports::services_customers_report');
    $routes->get('services/sales-report', 'Admin\Reports::services_sales_report');
});

$routes->group('seller', ['filter' => 'role:Seller'], function ($routes) {
    $routes->get('/', 'Seller\Home::index');
    $routes->get('dashboard', 'Seller\Home::index');
    // Orders
    $routes->get('orders', 'Seller\Orders::index');
    $routes->get('finished-orders', 'Seller\Orders::finished_orders');
    $routes->get('pending-orders', 'Seller\Orders::pending_orders');
    $routes->get('cancelled-orders', 'Seller\Orders::cancelled_orders');
    $routes->match(["get", "post"], "product_invoice/(:num)", "Seller\Orders::product_invoice/$1");
    $routes->match(["get", "post"], "cancel_order/(:num)", "Seller\Orders::cancel_order/$1");
    $routes->match(["get", "post"], "confirm_order/(:num)", "Seller\Orders::confirm_order/$1");
    // Service Orders
    $routes->get('service-orders', 'Seller\Orders::service_orders');
    $routes->get('completed-service-orders', 'Seller\Orders::completed_service_orders');
    $routes->get('pending-service-orders', 'Seller\Orders::pending_service_orders');
    $routes->get('cancelled-service-orders', 'Seller\Orders::cancelled_service_orders');
    $routes->match(["get", "post"], "service_invoice/(:num)", "Seller\Orders::service_invoice/$1");
    $routes->match(["get", "post"], "cancel_service_order/(:num)", "Seller\Orders::cancel_service_order/$1");
    $routes->match(["get", "post"], "confirm_service_order/(:num)", "Seller\Orders::confirm_service_order/$1");
    // Products
    $routes->get('products', 'Seller\Products::index');
    $routes->match(["get", "post"], "add-products", "Seller\Products::add_products");
    $routes->match(["get", "post"], "edit_product/(:num)", "Seller\Products::edit_product/$1");
    $routes->get('banned-products', 'Seller\Products::banned_products');
    $routes->get('delete_product/(:num)', 'Seller\Products::delete_product/$1');
    // Services
    $routes->get('services', 'Seller\Services::index');
    $routes->match(["get", "post"], "add-services", "Seller\Services::add_services");
    $routes->match(["get", "post"], "edit_service/(:num)", "Seller\Services::edit_service/$1");
    $routes->get('banned-services', 'Seller\Services::banned_services');
    $routes->get('delete_service/(:num)', 'Seller\Services::delete_service/$1');
    // Requests
    $routes->get('product-requests', 'Seller\Requests::index');
    $routes->match(["get", "post"], "view_request/(:num)", "Seller\Requests::view_request/$1");
    $routes->match(["get", "post"], "confirm_product_request/(:num)", "Seller\Requests::confirm_product_request/$1");
    $routes->match(["get", "post"], "cancel_product_request/(:num)", "Seller\Requests::cancel_product_request/$1");
    // Reviews
    $routes->get('product-reviews', 'Seller\Products::product_reviews');
    // $routes->post('product-reviews', 'Seller\Products::product_reviews/$1');
    $routes->match(["get", "post"], "review-search", "Seller\Products::review_search");
    // Settings;
    $routes->match(["get", "post"], "settings/(:num)", "Seller\Settings::settings/$1");
});

$routes->group('api', ['namespace' => 'App\Controllers\Api\Web'], function ($routes) {
    // General Routes
    $routes->post('register', 'Authentication::register');
    $routes->post('login', 'Authentication::login');
    $routes->get('profile', 'Users::profile');
    // Customer Routes
    $routes->post('customer/settings/account', 'Users::update_account');
    $routes->post('customer/settings/delivery', 'Users::update_delivery');
    $routes->post('customer/place-order', 'Users::placeOrder');
    $routes->post('customer/place-service-order', 'Users::placeServiceOrder');
    $routes->get('customer/product-orders', 'Users::product_order');
    $routes->get('customer/service-orders', 'Users::service_orders');
    $routes->get('customer/requests', 'Users::order_requests');
    // Seller Routes
    $routes->match(['get', 'post'], 'seller/products', 'Sellers::products');
    $routes->match(['post', 'delete'], 'seller/product-update/(:any)', 'Sellers::product_update/$1');
    $routes->match(['get', 'post'], 'seller/services', 'Sellers::services');
    $routes->match(['post', 'delete'], 'seller/service-update/(:any)', 'Sellers::service_update/$1');
    $routes->get('seller/product-orders', 'Sellers::product_orders');
    $routes->get('seller/service-orders', 'Sellers::service_orders');
    $routes->get('seller/requests', 'Sellers::requests');
    $routes->post('seller/request-update/(:any)', 'Sellers::request_update/$1');
    $routes->get('seller/product-reviews/(:any)', 'Sellers::product_reviews/$1');
    $routes->get('seller/service-reviews/(:any)', 'Sellers::service_reviews/$1');
    // Categories
    $routes->get('product-categories', 'Sellers::listProductCategories');
    $routes->get('product-units', 'Sellers::listProductUnits');
    $routes->get('service-categories', 'Sellers::listServiceCategories');
    $routes->get('service-units', 'Sellers::listServiceUnits');
    $routes->get('districts', 'Sellers::listDistricts');

    // Complaints
    $routes->post('customer/complain', 'Users::send_complain');
    // Requests
    $routes->post('customer/custom-request', 'Users::send_custom_request');
    // Review Product
    $routes->post('customer/review_product', 'Users::review_product');
    $routes->post('customer/review_service', 'Users::review_service');

    // Sellers
    $routes->get('home-sellers', 'Home::listSellers');
    $routes->get('seller/(:num)', 'Home::sellerProfile/$1');
    $routes->get('products/seller/(:num)', 'Home::sellerProducts/$1');
    // Markets
    $routes->get('home-markets', 'Home::listMarkets');
    $routes->get('products/market/(:num)', 'Home::listMarketProducts/$1');
    // Products
    $routes->get('home-products', 'Home::listProducts');
    $routes->get('product/(:num)', 'Home::productDetails/$1');
    $routes->get('reviews/product/(:num)', 'Home::productReviews/$1');
    // Services
    $routes->get('home-services', 'Home::listServices');
    $routes->get('service/(:num)', 'Home::serviceDetails/$1');
    $routes->get('reviews/service/(:num)', 'Home::serviceReviews/$1');
});


/*
 * --------------------------------------------------------------------
 * Additional Routing
 * --------------------------------------------------------------------
 *
 * There will often be times that you need additional routing and you
 * need it to be able to override any defaults in this file. Environment
 * based routes is one such time. require() additional route files here
 * to make that happen.
 *
 * You will have access to the $routes object within that file without
 * needing to reload it.
 */
if (is_file(APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php')) {
    require APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php';
}
