<?php

namespace App\Controllers\Admin;

use App\Controllers\BaseController;
use App\Models\Admin\Categories;
use App\Models\Admin\UnitsModel;
use App\Models\Admin\AttributesModel;
use App\Models\Admin\ServiceTypesModel;
use App\Models\Admin\ProductsModel;
use App\Models\Admin\ServicesModel;

class Marketplace extends BaseController
{
    private $categories;
    private $units;
    private $attributes;
    private $service_types;
    private $products;
    private $services;
    protected $table = 'product_reviews';
    protected $service_table = 'service_reviews';
    public $db;
    public $session;

    public function __construct()
    {
        $this->categories = new Categories();
        $this->units = new UnitsModel();
        $this->attributes = new AttributesModel();
        $this->service_types = new ServiceTypesModel();
        $this->products = new ProductsModel();
        $this->services = new ServicesModel();
        $this->db = \Config\Database::connect();
        $this->session = session();
    }

    // Market Products
    public function products()
    {
        $data['breadcrumb'] = 'Products';
        $data['products'] = $this->products->getProdutcs();
        return view('admin/marketplace/product_list', $data);
    }

    // Product Details
    public function product_details($id)
    {
        $data['breadcrumb'] = 'Product Details';
        $data['product'] = $this->products->productDetails($id);
        $data['countRating'] = $this->products->countRatings($id);
        $data['sumRating'] = $this->db->table($this->table)->where(['reviewed_product' => $id])->selectSum('review_rating', 'rating')->get()->getRow()->rating;
        $data['reviews'] = $this->products->product_reviews($id);
        return view('admin/marketplace/product_details', $data);
    }

    // Market Services
    public function services()
    {
        $data['breadcrumb'] = 'Services';
        $data['services'] = $this->services->getServices();
        return view('admin/marketplace/service_list', $data);
    }

    // Service Details
    public function service_details($id)
    {
        $data['breadcrumb'] = 'Service Details';
        $data['service'] = $this->services->serviceDetails($id);
        $data['countRating'] = $this->services->countRatings($id);
        $data['sumRating'] = $this->db->table($this->service_table)->where(['reviewed_service' => $id])->selectSum('review_rating', 'rating')->get()->getRow()->rating;
        $data['reviews'] = $this->services->service_reviews($id);
        return view('admin/marketplace/service_details', $data);
    }

    // Product Reviews
    public function reviews()
    {
        $data['breadcrumb'] = 'Reviews';
        $query = $this->db->query("SELECT * FROM product_reviews as r INNER JOIN products as p ON p.product_id=r.reviewed_product INNER JOIN users as u ON u.id=r.reviewer_id GROUP BY reviewed_product ORDER BY review_id DESC");
        $data['reviews'] = $query->getResult();
        return view('admin/marketplace/reviews', $data);
    }

    public function product_reviews($id)
    {
        $data['breadcrumb'] = 'Product Reviews';
        $data['reviews'] = $this->products->product_reviews($id);
        return view('admin/marketplace/product_reviews', $data);
    }

    public function product_invoice($id)
    {
        $data['breadcrumb'] = 'Invoice Details';
        $data['orders'] = $this->products->getOrderInvoice($id);
        return view('admin/marketplace/invoice_details', $data);
    }

    public function service_invoice($id)
    {
        $data['breadcrumb'] = 'Invoice Details';
        $data['orders'] = $this->services->getServiceOrderInvoice($id);
        return view('admin/marketplace/service_invoice', $data);
    }

    // Categories
    public function categories()
    {
        $data['breadcrumb'] = 'Categories';
        $data['categories'] = $this->categories->getCategories();
        return view('admin/attributes/categories', $data);
    }
    public function add_category()
    {
        $data['breadcrumb'] = 'Add Categories';
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'is_unique[categories.category_name]',
                'image' => 'uploaded[image]|max_size[image,120]|mime_in[image,image/png,image/jpg,image/gif]'
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'Category Name is alreay used'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'category_name' => $this->request->getVar('name'),
                    'category_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'category_thumb' => $new_image,
                ];

                $result = $this->categories->save($data);

                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/products/category/thumbs', $new_image);
                    }
                    $this->session->setTempdata('success', 'Category added successfully', 3);
                    return redirect()->to(site_url('admin/add-categories'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Category could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-categories'));
                }
            }
        }
        return view('admin/attributes/add_categories', $data);
    }

    public function edit_category($id)
    {
        $data['breadcrumb'] = 'Edit Category';
        $data['category'] = $this->categories->find($id);

        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'Category Name is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    // Unlink old image
                    $findFile = $this->categories->getCategoryID($id);
                    $old_img = $findFile->category_thumb;
                    $imagePath = 'public/uploads/products/category/thumbs/' . $old_img;

                    if (file_exists($old_img)) {
                        unlink('public/uploads/products/category/thumbs/' . $old_img);
                    }

                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'category_name' => $this->request->getVar('name'),
                    'category_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'category_thumb' => (!empty($_FILES['image']['name']) ? $new_image : $this->request->getVar('old_image')),
                ];

                $result = $this->categories->update($id, $data);

                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/products/category/thumbs', $new_image);
                    }
                    $this->session->setTempdata('success', 'Category updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Category could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }

        return view('admin/attributes/edit_category', $data);
    }

    public function delete_category($id)
    {

        // Unlink old image
        $findFile = $this->categories->getCategoryID($id);
        $old_img = $findFile->category_thumb;
        $imagePath = 'public/uploads/products/category/thumbs/' . $old_img;

        if (file_exists($old_img)) {
            unlink('public/uploads/products/category/thumbs/' . $old_img);
        }

        $result = $this->categories->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Category Deleted successfully', 3);
            return redirect()->to(site_url('admin/categories'));
        }
    }

    // Attributes
    public function attributes()
    {
        $data['breadcrumb'] = 'Attributes';
        $data['attributes'] = $this->attributes->getAttributes();
        return view('admin/attributes/attributes', $data);
    }
    public function add_attributes()
    {
        $data['breadcrumb'] = 'Add Attributes';
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'is_unique[attributes.attribute_name]',
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'Attribute Name is alreay used'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'attribute_name' => ucwords($this->request->getVar('name')),
                ];

                $result = $this->attributes->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Attribute added successfully', 3);
                    return redirect()->to(site_url('admin/add-attributes'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Attribute could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-attributes'));
                }
            }
        }
        return view('admin/attributes/add_attribute', $data);
    }
    public function edit_attribute($id)
    {
        $data['breadcrumb'] = 'Edit Attribute';
        $data['attribute'] = $this->attributes->find($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'Attribute Name is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'attribute_name' => ucwords($this->request->getVar('name')),
                ];

                $result = $this->attributes->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Attribute updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Attribute could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/attributes/edit_attribute', $data);
    }
    public function delete_attribute($id)
    {
        $result = $this->attributes->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Attribute Deleted successfully', 3);
            return redirect()->to(site_url('admin/attributes'));
        } else {
            $this->session->setTempdata('error', 'Attribute couldnot be deleted', 3);
            return redirect()->to(site_url('admin/attributes'));
        }
    }

    // Units
    public function units()
    {
        $data['breadcrumb'] = 'Units';
        $data['units'] = $this->units->getUnits();
        return view('admin/attributes/units', $data);
    }

    public function add_units()
    {
        $data['breadcrumb'] = 'Add Units';
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'is_unique[units.unit_name]',
                'short_name' => 'is_unique[units.unit_shortname]'
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'Unit Name is alreay used'
                ],
                'short_name' => [
                    'is_unique' => 'Unit Short Name is alreay used'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'unit_name' => ucwords($this->request->getVar('name')),
                    'unit_shortname' => strtolower($this->request->getVar('short_name')),
                ];

                $result = $this->units->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Unit added successfully', 3);
                    return redirect()->to(site_url('admin/add-units'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Unit could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-units'));
                }
            }
        }
        return view('admin/attributes/add_units', $data);
    }

    public function edit_unit($id)
    {
        $data['breadcrumb'] = 'Edit Unit';
        $data['unit'] = $this->units->find($id);

        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
                'short_name' => 'required'
            ];

            $messages = [
                'name' => [
                    'required' => 'Unit Name is required'
                ],
                'short_name' => [
                    'required' => 'Unit Short Name is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'unit_name' => ucwords($this->request->getVar('name')),
                    'unit_shortname' => strtolower($this->request->getVar('short_name')),
                ];

                $result = $this->units->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Unit updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Unit could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }

        return view('admin/attributes/edit_unit', $data);
    }

    public function delete_unit($id)
    {
        $result = $this->units->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Unit Deleted successfully', 3);
            return redirect()->to(site_url('admin/units'));
        } else {
            $this->session->setTempdata('error', 'Unit couldnot be deleted', 3);
            return redirect()->to(site_url('admin/units'));
        }
    }

    // Service Types
    public function service_types()
    {
        $data['breadcrumb'] = 'Service Types';
        $data['types'] = $this->service_types->getServiceTypes();
        return view('admin/attributes/service_types', $data);
    }
    public function add_service_type()
    {
        $data['breadcrumb'] = 'Add Service Type';
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'is_unique[service_types.service_type_name]',
                'image' => 'uploaded[image]|max_size[image,120]|mime_in[image,image/png,image/jpg,image/gif]|max_dims[image,120,120]'
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'Service Type Name is alreay used'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'service_type_name' => ucwords($this->request->getVar('name')),
                    'service_type_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'service_type_thumb' => $new_image,
                ];

                $result = $this->service_types->save($data);

                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/services/types/thumbs', $new_image);
                    }
                    $this->session->setTempdata('success', 'Service Type added successfully', 3);
                    return redirect()->to(site_url('admin/add-service-type'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Service Type could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-service-type'));
                }
            }
        }
        return view('admin/attributes/add_service_type', $data);
    }
    public function edit_service_type($id)
    {
        $data['breadcrumb'] = 'Edit Service Type';
        $data['service_type'] = $this->service_types->find($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'Service Type Name is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['image']['name'])) {
                    // Unlink old image
                    $findFile = $this->service_types->getServiceTypeID($id);
                    $old_img = $findFile->service_type_thumb;
                    $imagePath = 'public/uploads/services/types/thumbs/' . $old_img;

                    if (file_exists($old_img)) {
                        unlink('public/uploads/services/types/thumbs/' . $old_img);
                    }

                    $image = $this->request->getFile('image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'service_type_name' => $this->request->getVar('name'),
                    'service_type_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'service_type_thumb' => (!empty($_FILES['image']['name']) ? $new_image : $this->request->getVar('old_image')),
                ];

                $result = $this->service_types->update($id, $data);

                if ($result) {
                    if (!empty($_FILES['image']['name'])) {
                        $image->move('public/uploads/services/types/thumbs', $new_image);
                    }
                    $this->session->setTempdata('success', 'Service Type updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Service Type could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/attributes/edit_service_type', $data);
    }

    public function delete_service_type($id)
    {
        // Unlink old image
        $findFile = $this->service_types->getServiceTypeID($id);
        $old_img = $findFile->service_type_thumb;
        $imagePath = 'public/uploads/services/types/thumbs/' . $old_img;

        if (file_exists($old_img)) {
            unlink('public/uploads/services/types/thumbs/' . $old_img);
        }

        $result = $this->service_types->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Service Type Deleted successfully', 3);
            return redirect()->to(site_url('admin/service-types'));
        }
    }

    // Product Requests
    public function product_requests()
    {
        $data['breadcrumb'] = 'Product Requests';
        return view('admin/marketplace/product_requests', $data);
    }
}
