<?php

namespace App\Controllers\Admin;

use App\Controllers\BaseController;
use App\Models\Admin\MarketsModel;
use App\Models\Admin\DistrictsModel;
use App\Models\Admin\EmailSettingsModel;
use App\Models\Admin\OrganSettingsModel;
use Myth\Auth\Config\Auth as AuthConfig;
use App\Models\Web\UserModel;

class SoftSettings extends BaseController
{
    private $markets;
    public $districts;
    public $email_settings;
    public $organ_settings;
    public $users;
    protected $auth;
    protected $config;
    public $session;
    public $db;

    public function __construct()
    {
        $this->markets = new MarketsModel();
        $this->districts = new DistrictsModel();
        $this->email_settings = new EmailSettingsModel();
        $this->organ_settings = new OrganSettingsModel();
        $this->users = new UserModel();
        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->session = session();
        $this->db = \Config\Database::connect();
    }

    // Email Settings
    public function email_settings()
    {

        $data['breadcrumb'] = 'Email Settings';
        $data['settings'] = $this->email_settings->getSettings();
        return view('admin/soft_settings/email_settings', $data);
    }

    public function update_emailsettings($id)
    {

        if ($this->request->getMethod() == 'post') {
            $rules = [
                'email' => 'required|valid_email',
                'names' => 'required',
                'protocol' => 'required',
                'host' => 'required',
                'port' => 'required',
                'encryption' => 'required',
                'mail_type' => 'required'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {
                $old_pass = $this->request->getVar('old_pass');
                $data = [
                    'email' => $this->request->getVar('email'),
                    'password' => (!empty($this->request->getVar('password')) ? $this->request->getVar('password') : $old_pass),
                    'names' => $this->request->getVar('names'),
                    'smtp_protocol' => $this->request->getVar('protocol'),
                    'smtp_host' => $this->request->getVar('host'),
                    'smtp_port' => $this->request->getVar('port'),
                    'smtp_encryption' => $this->request->getVar('encryption'),
                    'mail_type' => $this->request->getVar('mail_type'),
                ];

                $result = $this->email_settings->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Settings updated successfully', 3);
                    return redirect()->to(site_url('admin/email-settings'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Settings could not be updated, try again', 3);
                    return redirect()->to(site_url('admin/email-settings'));
                }
            }
        }
    }

    // Organization Settings
    public function organ_settings()
    {
        $data['breadcrumb'] = 'Email Settings';
        $query = $this->db->query("SELECT * FROM organ_settings");
        $data['settings'] = $query->getRow();
        return view('admin/soft_settings/organ_settings', $data);
    }

    // Account settings
    public function account_settings()
    {

        $data['breadcrumb'] = 'Account Settings';
        if ($this->request->getMethod() == 'post') {

            $rules = [
                'password' => 'required|min_length[8]',
                'confirm_password' => 'required|matches[password]',
            ];

            $messages = [
                'password' => [
                    'required' => 'Password is required',
                    'min_length' => 'Password should have 8 or more characters'
                ],
                'confirm_password' => [
                    'required' => 'Confirm password is required',
                    'matches' => 'The two passwords donot match'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $config = config('Auth');

                if (
                    (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                    ||
                    (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
                ) {
                    $hashOptions = [
                        'memory_cost' => $config->hashMemoryCost,
                        'time_cost'   => $config->hashTimeCost,
                        'threads'     => $config->hashThreads,
                    ];
                } else {
                    $hashOptions = [
                        'cost' => $config->hashCost,
                    ];
                }

                $password = $this->request->getVar('password');

                $data = [
                    'password_hash' => password_hash(self::preparePassword($password), $this->config->hashAlgorithm, $hashOptions),
                ];

                $result = $this->users->update(user()->id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Password updated successfully', 3);
                    return redirect()->to('admin/account-settings');
                } else {
                    $this->session->setTempdata('erro', 'Password couldnot be updated', 3);
                    return redirect()->to('admin/account-settings');
                }
            }
        }

        return view('admin/soft_settings/account_settings', $data);
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }

    public function update_organsettings($id)
    {

        if ($this->request->getMethod() == 'post') {

            $rules = [
                'footer_text' => 'required',
                'contact_info' => 'required',
                'emails' => 'required',
                'mobiles' => 'required',
                'copyright' => 'required',
            ];

            $messages = [
                'footer_text' => [
                    'required' => 'Footer text is required',
                ],
                'contact_info' => [
                    'required' => 'Contact info is required',
                ],
                'emails' => [
                    'required' => 'Company emails is required',
                ],
                'mobiles' => [
                    'required' => 'Company mobiles is required',
                ],
                'copyright' => [
                    'required' => 'Copyright is required',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $query = $this->db->query("SELECT * FROM organ_settings");
                $result = $query->getRow();

                if (!empty($_FILES['logo']['name'])) {
                    // Unlink old image
                    $old_logo = $result->logo;
                    $imagePath = 'public/uploads/template/' . $old_logo;

                    if (file_exists($old_logo)) {
                        unlink('public/uploads/template/' . $old_logo);
                    }
                    $logo = $this->request->getFile('logo');
                    $new_logo = $logo->getRandomName();
                }

                if (!empty($_FILES['favicon']['name'])) {
                    // Unlink old image
                    $old_favicon = $result->favicon;
                    $imagePath = 'public/uploads/template/' . $old_favicon;

                    if (file_exists($old_favicon)) {
                        unlink('public/uploads/template/' . $old_favicon);
                    }
                    $favicon = $this->request->getFile('favicon');
                    $new_favicon = $favicon->getRandomName();
                }

                $old_logo = $this->request->getVar('old_logo');
                $old_favicon = $this->request->getVar('old_favicon');

                $data = [
                    'logo' => (!empty($_FILES['logo']['name']) ? $new_logo : $old_logo),
                    'favicon' => (!empty($_FILES['favicon']['name']) ? $new_favicon : $old_favicon),
                    'footer_text' => $this->request->getVar('footer_text'),
                    'contact_info' => $this->request->getVar('contact_info'),
                    'emails' => $this->request->getVar('emails'),
                    'mobiles' => $this->request->getVar('mobiles'),
                    'copyright' => $this->request->getVar('copyright'),
                ];

                $result = $this->organ_settings->update($id, $data);

                if ($result) {
                    if (!empty($_FILES['logo']['name'])) {
                        $logo->move('public/uploads/template', $new_logo);
                    }
                    if (!empty($_FILES['favicon']['name'])) {
                        $favicon->move('public/uploads/template', $new_favicon);
                    }
                    $this->session->setTempdata('success', 'Settings updated successfully', 3);
                    return redirect()->to(site_url('admin/organ-settings'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Settings could not be updated, try again', 3);
                    return redirect()->to(site_url('admin/organ-settings'));
                }
            }
        }
    }

    // Districts
    public function districts()
    {
        $data['breadcrumb'] = 'Districts';
        $data['districts'] = $this->districts->getDistricts();
        return view('admin/soft_settings/districts', $data);
    }
    public function add_districts()
    {
        $data['breadcrumb'] = 'Add District';
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required|is_unique[markets.market_name]',
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'District Name is alreay used',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'district_name' => ucwords($this->request->getVar('name')),
                    'district_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                ];

                $result = $this->districts->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'District added successfully', 3);
                    return redirect()->to(site_url('admin/add-districts'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! District could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-districts'));
                }
            }
        }
        return view('admin/soft_settings/add_district', $data);
    }

    public function edit_district($id)
    {
        $data['breadcrumb'] = 'Edit District';
        $data['district'] = $this->districts->find($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'District Name is required',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'district_name' => ucwords($this->request->getVar('name')),
                    'district_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                ];

                $result = $this->districts->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'District updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! District could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/soft_settings/edit_district', $data);
    }

    // Markets
    public function markets()
    {
        $data['breadcrumb'] = 'Markets';
        $data['markets'] = $this->markets->getMarkets();
        return view('admin/soft_settings/markets', $data);
    }
    public function add_markets()
    {

        $data['breadcrumb'] = 'Add Market';
        $data['districts'] = $this->districts->getDistricts();
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required|is_unique[markets.market_name]',
            ];

            $messages = [
                'name' => [
                    'is_unique' => 'Market Name is alreay used',
                    'required' => 'Market Name is required',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'market_name' => ucwords($this->request->getVar('name')),
                    'market_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'market_district' => $this->request->getVar('district'),
                ];

                $result = $this->markets->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Market added successfully', 3);
                    return redirect()->to(site_url('admin/add-markets'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Market could not be added, try again', 3);
                    return redirect()->to(site_url('admin/add-markets'));
                }
            }
        }
        return view('admin/soft_settings/add_markets', $data);
    }
    public function edit_market($id)
    {
        $data['breadcrumb'] = 'Edit Market';
        $data['districts'] = $this->districts->getDistricts();
        $data['market'] = $this->markets->getMarketId($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'Market Name is required',
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'market_name' => $this->request->getVar('name'),
                    'market_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'market_district' => $this->request->getVar('district'),
                ];

                $result = $this->markets->update($id, $data);

                if ($result) {
                    $this->session->setTempdata('success', 'Market updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Market could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('admin/soft_settings/edit_market', $data);
    }
    public function delete_market($id)
    {
        $result = $this->markets->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Market Deleted successfully', 3);
            return redirect()->to(site_url('admin/markets'));
        }
    }
}
