<?php

namespace App\Controllers\Api\Web;

use CodeIgniter\RESTful\ResourceController;
use App\Controllers\BaseController;
use Myth\Auth\Config\Auth as AuthConfig;
use Myth\Auth\Entities\User;
use Myth\Auth\Models\UserModel;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use Exception;

class Authentication extends ResourceController
{
    protected $format = 'json';
    protected $auth;
    protected $config;
    protected $session;
    public $user;
    public $db;

    public function __construct()
    {
        // Most services in this controller require
        // the session to be started - so fire it up!
        $this->session = service('session');

        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->user = new UserModel();
        $this->db = \Config\Database::connect();
    }

    // Register User
    public function register()
    {

        if ($this->request->getMethod() == 'post') {

            // Valid fields
            $rules = [
                'firstname' => 'required|min_length[3]',
                'lastname' => 'required|min_length[3]',
                'phone' => 'required|min_length[10]|is_unique[users.phone]',
                'category' => 'required',
                'address' => 'required',
                'district' => 'required',
                'password' => 'required|min_length[8]',
                'confirm_password' => 'required|matches[password]',
            ];

            // Messages
            $messages = [
                'firstname' => [
                    'required' => 'First name is required'
                ],
                'lastname' => [
                    'required' => 'Last name is required'
                ],
                'phone' => [
                    'required' => 'Mobile number is required',
                    'min_length' => 'Mobile number should not be less than 10 characters',
                ],
                'category' => [
                    'required' => 'Please select user type',
                ],
                'address' => [
                    'required' => 'Your address is required'
                ],
                'district' => [
                    'required' => 'Your district is required'
                ],
                'password' => [
                    'required' => 'Please set your password',
                    'min_length' => 'Password must be atleast 8 characters long'
                ],
                'confirm_password' => [
                    'required' => 'Confirm password is required',
                    'matches' => 'The two passwords donot match'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    // If no errors
                    $users = model(UserModel::class);
                    $config = config('Auth');
                    if (
                        (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                        ||
                        (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
                    ) {
                        $hashOptions = [
                            'memory_cost' => $config->hashMemoryCost,
                            'time_cost'   => $config->hashTimeCost,
                            'threads'     => $config->hashThreads,
                        ];
                    } else {
                        $hashOptions = [
                            'cost' => $config->hashCost,
                        ];
                    }

                    $category = $this->request->getVar('category');

                    $data = [
                        'firstname' => $this->request->getVar('firstname'),
                        'lastname' => $this->request->getVar('lastname'),
                        'phone' => $this->request->getVar('phone'),
                        'email' => $this->request->getVar('email'),
                        'category' => $category,
                        'address' => $this->request->getVar('address'),
                        'district' => $this->request->getVar('district'),
                        'password_hash' => password_hash(self::preparePassword($this->request->getVar('password')), $this->config->hashAlgorithm, $hashOptions),
                        'active' => 1,
                        'verification' => 1
                    ];

                    // Save the user
                    if ($users->insert($data)) {
                        $user_id = $users->getInsertID();
                        if ($category == 2) {
                            $user_group = 2;
                        } else {
                            $user_group = 3;
                        }
                        $this->db->query("INSERT INTO auth_groups_users(group_id, user_id)VALUES($user_group, $user_id)");
                        // Success
                        $response = [
                            "status" => 200,
                            "message" => 'User Registration successfull',
                            "error" => false,
                            "data" => [],
                        ];
                    } else {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Registration failed',
                            "error" => true,
                            "data" => [],
                        ];
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }

            return $this->respondCreated($response);
        }
    }

    // Login User
    public function login()
    {

        // Check request
        if ($this->request->getMethod() == 'post') {

            $rules = [
                "phone" => "required|min_length[10]",
                "password" => "required|min_length[8]",
            ];

            $messages = [
                'phone' => [
                    'required' => 'Phone number is required',
                    'min_length' => 'Phone can\'t be less than 10 characters'
                ],
                'password' => [
                    'required' => 'Password is required',
                    'min_length' => 'Password can\'t be less than 8 characters'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                // Error
                $response = [
                    "status" => 500,
                    "message" => $this->validator->getErrors(),
                    "error" => true,
                    "data" => [],
                ];
            } else {

                try {
                    // No errrors
                    $login = $this->request->getPost('phone');
                    $password = $this->request->getPost('password');
                    $remember = (bool) $this->request->getPost('remember');

                    // Determine credential type
                    $type = $login ? 'phone' : 'username';

                    $userdata = $this->user->where("phone", $login)->first();

                    if (!empty($userdata)) {
                        $query = $this->db->query("SELECT category FROM users WHERE phone='$login'");
                        $result = $query->getRow();
                        if ($result->category == 1) {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => "Please login with customer or seller account",
                                "error" => true,
                                "data" => [],
                            ];
                        } elseif ($result->category == 2) {
                            // Try to log them in...
                            if (!$this->auth->attempt([$type => $login, 'password' => $password], $remember)) {
                                // Error
                                $response = [
                                    "status" => 500,
                                    "message" => $this->auth->error() ?? lang('Auth.badAttempt'),
                                    "error" => true,
                                    "data" => [],
                                ];
                            } else {

                                $iat = time();
                                $nbf = $iat;
                                $exp = $iat + 604800;

                                $payload = [
                                    // 'iss' => 'amisug.org',
                                    // 'aud' => 'amisug.org',
                                    // 'iat' => 1356999524,
                                    // 'nbf' => 1357000000
                                    "iat" => $iat,
                                    "nbf" => $nbf,
                                    "exp" => $exp,
                                    "userdata" => $userdata
                                ];

                                $token = JWT::encode($payload, $this->getKey(), 'HS256', $userdata->password);

                                $district = $this->user->getUserDistrict($userdata->district);
                                $market = $this->user->getUserMarket($userdata->market);

                                $response = [
                                    "status" => 200,
                                    "message" => "User loggedin",
                                    "error" => false,
                                    "data" => [
                                        "token" => $token,
                                        "user_id" => $userdata->id,
                                        "firstname" => $userdata->firstname,
                                        "lastname" => $userdata->lastname,
                                        "store_name" => $userdata->store_name,
                                        "store_tagline" => $userdata->store_tagline,
                                        "store_description" => $userdata->store_description,
                                        "category" => $userdata->category,
                                        "market" => $market->market_name,
                                        "slug" => $userdata->slug,
                                        "phone" => $userdata->phone,
                                        "email" => $userdata->email,
                                        "address" => $userdata->address,
                                        "district" => $district->district_name,
                                        "city" => $userdata->city,
                                        "profile_image" => $userdata->profile_image,
                                    ],
                                ];
                            }
                        } else {
                            // Try to log them in...
                            if (!$this->auth->attempt([$type => $login, 'password' => $password], $remember)) {
                                // Error
                                $response = [
                                    "status" => 500,
                                    "message" => $this->auth->error() ?? lang('Auth.badAttempt'),
                                    "error" => true,
                                    "data" => [],
                                ];
                            } else {

                                $iat = time();
                                $nbf = $iat;
                                $exp = $iat + 604800;

                                $payload = [
                                    "iat" => $iat,
                                    "nbf" => $nbf,
                                    "exp" => $exp,
                                    "userdata" => $userdata
                                ];

                                $token = JWT::encode($payload, $this->getKey(), 'HS256', $userdata->password);

                                $district = $this->user->getUserDistrict($userdata->district);

                                $response = [
                                    "status" => 200,
                                    "message" => "User loggedin",
                                    "error" => false,
                                    "data" => [
                                        "token" => $token,
                                        "user_id" => $userdata->id,
                                        "firstname" => $userdata->firstname,
                                        "lastname" => $userdata->lastname,
                                        "dob" => $userdata->dob,
                                        "phone" => $userdata->phone,
                                        "email" => $userdata->email,
                                        "address" => $userdata->address,
                                        "district" => $district->district_name,
                                        "city" => $userdata->city,
                                        "profile_image" => $userdata->profile_image,
                                    ],
                                ];
                            }
                        }
                    } else {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => "Credentials not found",
                            "error" => true,
                            "data" => [],
                        ];
                    }
                } catch (Exception $ex) {
                    $response = [
                        "status" => 500,
                        "message" => $ex->getMessage(),
                        "error" => true,
                        "data" => [],
                    ];
                }
            }
        } else {
            // Error
            $response = [
                "status" => 500,
                "message" => 'Unknown Request',
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    public function getKey()
    {
        $key = "ADcdGdecDEVNK";
        return $key;
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }
}
