<?php

namespace App\Controllers\Api\Web;

use CodeIgniter\RESTful\ResourceController;
use App\Controllers\BaseController;
use Myth\Auth\Config\Auth as AuthConfig;
use Myth\Auth\Entities\User;
use Myth\Auth\Models\UserModel;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use Exception;
use App\Models\Vendor\ProductsModel;
use App\Models\Vendor\ServicesModel;
use App\Models\Api\Web\SellersModel;
use App\Models\Vendor\ProductRequestsModel;

class Sellers extends ResourceController
{

    protected $format = 'json';
    protected $auth;
    protected $config;
    protected $session;
    public $products_model;
    public $services_model;
    public $sellers;
    public $user;
    public $requests;
    public $db;

    public function __construct()
    {

        $this->products_model = new ProductsModel();
        $this->services_model = new ServicesModel();
        $this->sellers = new SellersModel();
        $this->requests = new ProductRequestsModel();
        // Most services in this controller require
        // the session to be started - so fire it up!
        $this->session = service('session');

        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->user = new UserModel();
        $this->db = \Config\Database::connect();
    }

    // Seller products
    public function products()
    {

        $authorize = $this->request->getHeader("Authorization");

        try {
            if ($authorize) {
                // No errors
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;
                $user_category = $decoded_data->userdata->category;

                // Deny admin accounts from placing orders
                if ($user_category == 1) {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                } elseif ($user_category == 2) {

                    if ($this->request->getMethod() == 'get') {
                        $order_data = $this->sellers->getSellerProducts($userID);
                        $response = [
                            "status" => 200,
                            "message" => 'Order List',
                            "error" => false,
                            "data" => $order_data,
                        ];
                    } elseif ($this->request->getMethod() == 'post') {
                        $rules = [
                            'product_name' => 'required',
                            'category'  => 'required',
                            'units'  => 'required',
                            'sale_type' => 'required',
                            'unit_value' => 'required',
                            'price' => 'required',
                            'description' => 'required',
                            'product_image' => 'uploaded[product_image]'
                        ];

                        $messages = [
                            'product_name' => [
                                'required' => 'Product name is required'
                            ],
                            'sale_type' => [
                                'required' => 'Please choose between wholesale or retail'
                            ],
                            'unit_value' => [
                                'required' => 'Product unit value is required'
                            ],
                            'description' => [
                                'required' => 'Product description is required'
                            ],
                            'product_image' => [
                                'uploaded' => 'Product image is required'
                            ],
                        ];

                        if (!$this->validate($rules, $messages)) {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => $this->validator->getErrors(),
                                "error" => true,
                                "data" => [],
                            ];
                        } else {

                            // No errors
                            $district = $decoded_data->userdata->district;
                            $market = $decoded_data->userdata->market;

                            // Deny admin accounts from placing orders
                            if ($user_category == 1) {
                                // Error
                                $response = [
                                    "status" => 500,
                                    "message" => 'Authorization Error !! Please register with a seller account',
                                    "error" => true,
                                    "data" => [],
                                ];
                            } elseif ($user_category == 2) {

                                helper('text');

                                $image = $this->request->getFile('product_image');
                                $new_image = $image->getRandomName();

                                $product_ref = random_string('alnum', 10);
                                $sku = "AG" . random_string('alnum', 5);

                                $data = [
                                    'product_ref' => $product_ref,
                                    'product_category' => $this->request->getVar('category'),
                                    'product_vendor' => $userID,
                                    'product_name' => ucwords($this->request->getVar('product_name')),
                                    'product_slug' => url_title(strtolower($this->request->getVar('product_name')), '-', true),
                                    'product_desc' => $this->request->getVar('description'),
                                    'product_image' => $new_image,
                                    'product_unit' => $this->request->getVar('units'),
                                    'sale_type' => $this->request->getVar('sale_type'),
                                    'unit_value' => $this->request->getVar('unit_value'),
                                    'sku' => $sku,
                                    'product_price' => $this->request->getVar('price'),
                                    'product_status' => 1,
                                    'product_district' => $district,
                                    'product_market' => $market,
                                ];

                                if ($this->products_model->insert($data)) {

                                    $image->move('public/uploads/products/products', $new_image);
                                    // Success
                                    $response = [
                                        "status" => 200,
                                        "message" => 'Product added successfully',
                                        "error" => false,
                                        "data" => [],
                                    ];
                                } else {
                                    // Error
                                    $response = [
                                        "status" => 500,
                                        "message" => 'Falied to add product',
                                        "error" => true,
                                        "data" => [],
                                    ];
                                }
                            } else {
                                // Error
                                $response = [
                                    "status" => 500,
                                    "message" => 'Authorization Error !! Please register with a seller account',
                                    "error" => true,
                                    "data" => [],
                                ];
                            }
                        }
                    } else {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Invalid request methods',
                            "error" => true,
                            "data" => [],
                        ];
                    }
                } else {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }
        return $this->respondCreated($response);
    }

    // Product update and Delete
    public function product_update($id)
    {

        $authorize = $this->request->getHeader("Authorization");

        try {
            if ($authorize) {

                // No errors
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;
                $user_category = $decoded_data->userdata->category;
                $district = $decoded_data->userdata->district;
                $market = $decoded_data->userdata->market;

                if ($this->request->getMethod() == 'post') {
                    $rules = [
                        'product_name' => 'required',
                        'category'  => 'required',
                        'units'  => 'required',
                        'sale_type' => 'required',
                        'unit_value' => 'required',
                        'price' => 'required',
                        'description' => 'required',
                    ];

                    $messages = [
                        'product_name' => [
                            'required' => 'Product name is required'
                        ],
                        'category' => [
                            'required' => 'Product category is required'
                        ],
                        'units' => [
                            'required' => 'Product unit is required'
                        ],
                        'sale_type' => [
                            'required' => 'Please choose between wholesale or retail'
                        ],
                        'unit_value' => [
                            'required' => 'Product unit value is required'
                        ],
                        'description' => [
                            'required' => 'Product description is required'
                        ],
                        'price' => [
                            'required' => 'Product price is required'
                        ],
                    ];

                    if (!$this->validate($rules, $messages)) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => $this->validator->getErrors(),
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        // Deny admin accounts from placing orders
                        if ($user_category == 1) {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Authorization Error !! Please register with a seller account',
                                "error" => true,
                                "data" => [],
                            ];
                        } elseif ($user_category == 2) {

                            helper('text');

                            $product_category = $this->request->getVar('category');
                            $product_name = ucwords($this->request->getVar('product_name'));
                            $product_slug = url_title(strtolower($this->request->getVar('product_name')), '-', true);
                            $product_desc = $this->request->getVar('description');
                            $product_unit = $this->request->getVar('units');
                            $sale_type = $this->request->getVar('sale_type');
                            $unit_value = $this->request->getVar('unit_value');
                            $product_price = $this->request->getVar('price');

                            if (!empty($_FILES['product_image']['name'])) {
                                // Unlink old image
                                $findFile = $this->products->getProductID($id);
                                $old_img = $findFile->product_image;
                                $imagePath = 'public/uploads/products/products/' . $old_img;

                                if (file_exists($imagePath)) {
                                    unlink('public/uploads/products/products/' . $old_img);
                                }

                                $image = $this->request->getFile('product_image');
                                $new_image = $image->getRandomName();

                                $query = $this->db->query("UPDATE products SET product_category=" . $this->db->escape($product_category) . ", product_name=" . $this->db->escape($product_name) . ", product_slug=" . $this->db->escape($product_slug) . ", product_desc=" . $this->db->escape($product_desc) . ", product_image=" . $this->db->escape($new_image) . ", product_unit=" . $this->db->escape($product_unit) . ", sale_type=" . $this->db->escape($sale_type) . ", unit_value=" . $this->db->escape($unit_value) . ", product_price=" . $this->db->escape($product_price) . " WHERE product_id='$id' AND product_vendor='$userID'");

                                if ($query) {

                                    if (!empty($_FILES['product_image']['name'])) {
                                        $image->move('public/uploads/products/products', $new_image);
                                    }
                                    // Success
                                    $response = [
                                        "status" => 200,
                                        "message" => 'Product updated successfully',
                                        "error" => false,
                                        "data" => [],
                                    ];
                                } else {
                                    // Error
                                    $response = [
                                        "status" => 500,
                                        "message" => 'Falied to update product',
                                        "error" => true,
                                        "data" => [],
                                    ];
                                }
                            } else {
                                $query = $this->db->query("UPDATE products SET product_category=" . $this->db->escape($product_category) . ", product_name=" . $this->db->escape($product_name) . ", product_slug=" . $this->db->escape($product_slug) . ", product_desc=" . $this->db->escape($product_desc) . ", product_unit=" . $this->db->escape($product_unit) . ", sale_type=" . $this->db->escape($sale_type) . ", unit_value=" . $this->db->escape($unit_value) . ", product_price=" . $this->db->escape($product_price) . " WHERE product_id='$id'");

                                if ($query) {
                                    // Success
                                    $response = [
                                        "status" => 200,
                                        "message" => 'Product updated successfully',
                                        "error" => false,
                                        "data" => [],
                                    ];
                                } else {
                                    // Error
                                    $response = [
                                        "status" => 500,
                                        "message" => 'Falied to update product',
                                        "error" => true,
                                        "data" => [],
                                    ];
                                }
                            }
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Authorization Error !! Please register with a seller account',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } elseif ($this->request->getMethod() == 'delete') {
                    // Unlink old image
                    $findFile = $this->products_model->getProductID($id);
                    $old_img = $findFile->product_image;
                    $imagePath = 'public/uploads/products/products/' . $old_img;

                    if (file_exists($imagePath)) {
                        unlink('public/uploads/products/products/' . $old_img);
                    }
                    $result = $this->products_model->delete($id);
                    if ($result) {

                        $response = [
                            "status" => 200,
                            "message" => 'Product Deleted successfully',
                            "error" => false,
                            "data" => [],
                        ];
                    } else {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => 'Failed to delete product',
                            "error" => true,
                            "data" => [],
                        ];
                    }
                } else {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Invalid request methods',
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Seller services
    public function services()
    {

        $authorize = $this->request->getHeader("Authorization");

        try {
            if ($authorize) {
                // No errors
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;
                $user_category = $decoded_data->userdata->category;

                // Deny admin accounts from placing orders
                if ($user_category == 1) {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                } elseif ($user_category == 2) {

                    if ($this->request->getMethod() == 'get') {
                        $order_data = $this->sellers->getSellerServices($userID);
                        $response = [
                            "status" => 200,
                            "message" => 'Order List',
                            "error" => false,
                            "data" => $order_data,
                        ];
                    } elseif ($this->request->getMethod() == 'post') {
                        $rules = [
                            'service_name' => 'required',
                            'category'  => 'required',
                            'units'  => 'required',
                            'price' => 'required',
                            'description' => 'required',
                            'service_image' => 'uploaded[service_image]',
                        ];

                        $messages = [
                            'service_name' => [
                                'required' => 'Service name is required'
                            ],
                            'description' => [
                                'required' => 'Service description is required'
                            ],
                        ];

                        if (!$this->validate($rules, $messages)) {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => $this->validator->getErrors(),
                                "error" => true,
                                "data" => [],
                            ];
                        } else {

                            $district = $decoded_data->userdata->district;

                            // Deny admin accounts from placing orders
                            if ($user_category == 1) {
                                // Error
                                $response = [
                                    "status" => 500,
                                    "message" => 'Authorization Error !! Please register with a seller account',
                                    "error" => true,
                                    "data" => [],
                                ];
                            } elseif ($user_category == 2) {

                                helper('text');

                                $image = $this->request->getFile('service_image');
                                $new_image = $image->getRandomName();

                                $service_ref = random_string('alnum', 10);
                                $sku = "AG" . random_string('alnum', 5);

                                $data = [
                                    'service_ref' => $service_ref,
                                    'service_category' => $this->request->getVar('category'),
                                    'service_vendor' => $userID,
                                    'service_name' => ucwords($this->request->getVar('service_name')),
                                    'service_slug' => url_title(strtolower($this->request->getVar('service_name')), '-', true),
                                    'service_desc' => $this->request->getVar('description'),
                                    'service_image' => $new_image,
                                    'service_unit' => $this->request->getVar('units'),
                                    'sku' => $sku,
                                    'service_price' => $this->request->getVar('price'),
                                    'service_district' => $district,
                                    'service_status' => 1,
                                ];

                                if ($this->services_model->insert($data)) {

                                    $image->move('public/uploads/services', $new_image);
                                    // Success
                                    $response = [
                                        "status" => 200,
                                        "message" => 'Service added successfully',
                                        "error" => false,
                                        "data" => [],
                                    ];
                                } else {
                                    // Error
                                    $response = [
                                        "status" => 500,
                                        "message" => 'Falied to add service',
                                        "error" => true,
                                        "data" => [],
                                    ];
                                }
                            } else {
                                // Error
                                $response = [
                                    "status" => 500,
                                    "message" => 'Authorization Error !! Please register with a seller account',
                                    "error" => true,
                                    "data" => [],
                                ];
                            }
                        }
                    }
                } else {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }
        return $this->respondCreated($response);
    }

    // Edit Service
    public function service_update($id)
    {

        try {

            $authorize = $this->request->getHeader("Authorization");

            if ($authorize) {

                // No errors
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;
                $user_category = $decoded_data->userdata->category;
                $district = $decoded_data->userdata->district;

                if ($this->request->getMethod() == 'post') {
                    $rules = [
                        'service_name' => 'required',
                        'category'  => 'required',
                        'units'  => 'required',
                        'price' => 'required',
                        'description' => 'required'
                    ];

                    $messages = [
                        'service_name' => [
                            'required' => 'Service name is required'
                        ],
                        'description' => [
                            'required' => 'Service description is required'
                        ],
                    ];

                    if (!$this->validate($rules, $messages)) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => $this->validator->getErrors(),
                            "error" => true,
                            "data" => [],
                        ];
                    } else {

                        // Deny admin accounts from placing orders
                        if ($user_category == 1) {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Authorization Error !! Please register with a seller account',
                                "error" => true,
                                "data" => [],
                            ];
                        } elseif ($user_category == 2) {

                            helper('text');

                            $service_category = $this->request->getVar('category');
                            $service_name = ucwords($this->request->getVar('service_name'));
                            $service_slug = url_title(strtolower($this->request->getVar('service_name')), '-', true);
                            $service_desc = $this->request->getVar('description');
                            $service_unit = $this->request->getVar('units');
                            $service_price = $this->request->getVar('price');

                            if (!empty($_FILES['service_image']['name'])) {
                                $findFile = $this->services_model->getServiceID($id);
                                $old_img = $findFile->service_image;
                                $imagePath = 'public/uploads/services/' . $old_img;
                                if (file_exists($imagePath)) {
                                    unlink('public/uploads/services/' . $old_img);
                                }

                                $image = $this->request->getFile('service_image');
                                $new_image = $image->getRandomName();

                                $query = $this->db->query("UPDATE services SET service_category=" . $this->db->escape($service_category) . ", service_name=" . $this->db->escape($service_name) . ", service_slug=" . $this->db->escape($service_slug) . ", service_desc=" . $this->db->escape($service_desc) . ", service_image=" . $this->db->escape($new_image) . ", service_unit=" . $this->db->escape($service_unit) . ", service_price=" . $this->db->escape($service_price) . ", service_district=" . $this->db->escape($district) . " WHERE service_id='$id' AND service_vendor='$userID'");

                                if ($query) {

                                    if (!empty($_FILES['service_image']['name'])) {
                                        $image->move('public/uploads/services', $new_image);
                                    }
                                    // Success
                                    $response = [
                                        "status" => 200,
                                        "message" => 'Service updated successfully',
                                        "error" => false,
                                        "data" => [],
                                    ];
                                } else {
                                    // Error
                                    $response = [
                                        "status" => 500,
                                        "message" => 'Falied to update service',
                                        "error" => true,
                                        "data" => [],
                                    ];
                                }
                            } else {
                                $query = $this->db->query("UPDATE services SET service_category=" . $this->db->escape($service_category) . ", service_name=" . $this->db->escape($service_name) . ", service_slug=" . $this->db->escape($service_slug) . ", service_desc=" . $this->db->escape($service_desc) . ", service_unit=" . $this->db->escape($service_unit) . ", service_price=" . $this->db->escape($service_price) . ", service_district=" . $this->db->escape($district) . " WHERE service_id='$id'");

                                if ($query) {
                                    // Success
                                    $response = [
                                        "status" => 200,
                                        "message" => 'Service updated successfully',
                                        "error" => false,
                                        "data" => [],
                                    ];
                                } else {
                                    // Error
                                    $response = [
                                        "status" => 500,
                                        "message" => 'Falied to update service',
                                        "error" => true,
                                        "data" => [],
                                    ];
                                }
                            }
                        } else {
                            // Error
                            $response = [
                                "status" => 500,
                                "message" => 'Authorization Error !! Please register with a seller account',
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                } elseif ($this->request->getMethod() == 'delete') {

                    // Unlink old image
                    $findFile = $this->services_model->getServiceID($id);
                    $old_img = $findFile->service_image;
                    $imagePath = 'public/uploads/services/' . $old_img;

                    if (file_exists($imagePath)) {
                        unlink('public/uploads/services/' . $old_img);
                    }

                    $result = $this->services_model->delete($id);

                    if ($result) {

                        $response = [
                            "status" => 200,
                            "message" => "Service deleted successfully",
                            "error" => false,
                            "data" => [],
                        ];
                    } else {
                        $response = [
                            "status" => 500,
                            "message" => "Failed to delete Service",
                            "error" => true,
                            "data" => [],
                        ];
                    }
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Product orders
    public function product_orders()
    {
        $authorize = $this->request->getHeader("Authorization");

        try {
            if ($authorize) {
                // No errors
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;
                $user_category = $decoded_data->userdata->category;

                // Deny admin accounts from placing orders
                if ($user_category == 1) {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                } elseif ($user_category == 2) {
                    $order_data = $this->sellers->getProductOrders($userID);
                    $response = [
                        "status" => 200,
                        "message" => 'Order List',
                        "error" => false,
                        "data" => $order_data,
                    ];
                } else {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }
        return $this->respondCreated($response);
    }

    // Service orders
    public function service_orders()
    {

        $authorize = $this->request->getHeader("Authorization");

        try {
            if ($authorize) {
                // No errors
                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;
                $user_category = $decoded_data->userdata->category;

                // Deny admin accounts from placing orders
                if ($user_category == 1) {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                } elseif ($user_category == 2) {
                    $order_data = $this->sellers->getServiceOrders($userID);
                    $response = [
                        "status" => 200,
                        "message" => 'Order List',
                        "error" => false,
                        "data" => $order_data,
                    ];
                } else {
                    // Error
                    $response = [
                        "status" => 500,
                        "message" => 'Authorization Error !! Please register with a seller account',
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }
        return $this->respondCreated($response);
    }

    // Seller requests
    public function requests()
    {
        $authorize = $this->request->getHeader("Authorization");
        try {
            if ($authorize) {

                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;

                $response = [
                    "status" => 200,
                    "message" => "Request Details",
                    "error" => true,
                    "data" => $this->sellers->getRequests($userID),
                ];
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Update requests
    public function request_update($id){

        $authorize = $this->request->getHeader("Authorization");

        try {
            
            if($authorize){

                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;

                if ($this->request->getMethod() == 'post') {

                    $rules = [
                        'status' => 'required',
                    ];
                    $messages = [
                        'status' => [
                            'required' => 'Please select invoice status'
                        ]
                    ];
                    if (!$this->validate($rules, $messages)) {
                        // Error
                        $response = [
                            "status" => 500,
                            "message" => $this->validator->getErrors(),
                            "error" => true,
                            "data" => [],
                        ];

                    } else {

                        $status = $this->request->getVar('status');

                        $data = [
                            'request_status' => $status,
                            'read_status' => 1,
                        ];

                        $result = $this->requests->update($id, $data);

                        if ($result) {

                            if($status == 1){
                                $message = "Request accepted successfully";
                                $response = [
                                    "status" => 200,
                                    "message" => $message,
                                    "error" => false,
                                    "data" => [],
                                ];
                            }elseif($status == 0){
                                $message = "Request Declined successfully";
                                $response = [
                                    "status" => 200,
                                    "message" => $message,
                                    "error" => false,
                                    "data" => [],
                                ];
                            }else{
                                $message = "Unknown request selection";
                                $response = [
                                    "status" => 500,
                                    "message" => $message,
                                    "error" => true,
                                    "data" => [],
                                ];
                            }

                        }else{
                            $response = [
                                "status" => 500,
                                "message" => "Failed to update invoice",
                                "error" => true,
                                "data" => [],
                            ];
                        }
                    }
                }

            }else{
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }

        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);

    }

    // Product reviews
    public function product_reviews($id){

        $authorize = $this->request->getHeader("Authorization");
        try {
            if ($authorize) {

                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;

                if($this->request->getMethod() == 'get'){
                    $response = [
                        "status" => 200,
                        "message" => "Product Reviews",
                        "error" => false,
                        "data" => $this->sellers->getProductReviews($userID, $id),
                    ];
                }else{
                    $response = [
                        "status" => 500,
                        "message" => "Invalid Request",
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Service reviews
    public function service_reviews($id){
        
        $authorize = $this->request->getHeader("Authorization");
        try {
            if ($authorize) {

                $token = $authorize->getValue();
                $decoded_data = JWT::decode($token, new Key($this->getKey(), "HS256"));

                $userID = $decoded_data->userdata->id;

                if($this->request->getMethod() == 'get'){
                    $response = [
                        "status" => 200,
                        "message" => "Service Reviews",
                        "error" => false,
                        "data" => $this->sellers->getServiceReviews($userID, $id),
                    ];
                }else{
                    $response = [
                        "status" => 500,
                        "message" => "Invalid Request",
                        "error" => true,
                        "data" => [],
                    ];
                }
            } else {
                $response = [
                    "status" => 500,
                    "message" => "User must login first",
                    "error" => true,
                    "data" => [],
                ];
            }
        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Product Categories
    public function listProductCategories(){

        try {

            $response = [
                "status" => 200,
                "message" => "Product Categories",
                "error" => true,
                "data" => $this->sellers->getCategories(),
            ];

        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Product Units
    public function listProductUnits(){

        try {

            $response = [
                "status" => 200,
                "message" => "Product Units",
                "error" => true,
                "data" => $this->sellers->getUnits(),
            ];

        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Service Categories
    public function listServiceCategories(){

        try {

            $response = [
                "status" => 200,
                "message" => "Service Categories",
                "error" => true,
                "data" => $this->sellers->getServiceTypes(),
            ];

        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);

    }

    public function listServiceUnits(){

        try {

            $response = [
                "status" => 200,
                "message" => "Service Units",
                "error" => true,
                "data" => $this->sellers->getServiceUnits(),
            ];

        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Districts
    public function listDistricts(){

        try {

            $response = [
                "status" => 200,
                "message" => "Districts",
                "error" => true,
                "data" => $this->sellers->getDistricts(),
            ];

        } catch (Exception $ex) {
            $response = [
                "status" => 500,
                "message" => $ex->getMessage(),
                "error" => true,
                "data" => [],
            ];
        }

        return $this->respondCreated($response);
    }

    // Key
    public function getKey()
    {
        $key = "ADcdGdecDEVNK";
        return $key;
    }

    // Password hashing
    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }
}
