<?php

namespace App\Controllers\Seller;

use App\Controllers\BaseController;
use App\Models\Vendor\ProductOrdersModel;
use App\Models\Vendor\ServiceOrdersModel;

class Orders extends BaseController
{
    public $orders;
    public $service_orders;
    public $session;
    public $db;

    public function __construct()
    {
        $this->orders = new ProductOrdersModel();
        $this->service_orders = new ServiceOrdersModel();
        $this->db = \Config\Database::connect();
        $this->session = session();
    }

    // Order list
    public function index()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Orders';
        $data['orders'] = $this->orders->getProductOrders($user_id);
        return view('vendors/orders/order_list', $data);
    }
    public function service_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Service Orders';
        $data['orders'] = $this->service_orders->getServiceOrders($user_id);
        return view('vendors/service-orders/service_order_list', $data);
    }

    // Product Invoice
    public function product_invoice($order_id)
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Invoice Details';
        $data['orders'] = $this->orders->getOrderInvoice($order_id);
        $this->db->query("UPDATE orders SET read_status=1 WHERE vendor_id='$user_id' AND order_id='$order_id'");
        return view('vendors/orders/invoice_details', $data);
    }
    public function service_invoice($order_id)
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Invoice Details';
        $data['orders'] = $this->service_orders->getServiceOrderInvoice($order_id);
        $this->db->query("UPDATE service_orders SET read_status=1 WHERE s_vendor_id='$user_id' AND s_order_id='$order_id'");
        return view('vendors/service-orders/service_invoice', $data);
    }
    // Cancel Invoice
    public function cancel_order($id)
    {
        $data['breadcrumb'] = 'Cancel Invoice';
        $data['orders'] = $this->orders->getOrderInvoice($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'status' => 'required',
            ];
            $messages = [
                'status' => [
                    'required' => 'Please select invoice status'
                ]
            ];
            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $data = [
                    'order_status' => 0,
                ];
                $result = $this->orders->update($id, $data);
                if ($result) {
                    $this->session->setTempdata('success', 'Order cancelled successfully', 3);
                    return redirect()->to(site_url('seller/orders'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Order could not be cancelled, try again', 3);
                    return redirect()->to(site_url('seller/orders'));
                }
            }
        }
        return view('vendors/orders/cancel_order', $data);
    }

    public function cancel_service_order($id)
    {
        $data['breadcrumb'] = 'Cancel Invoice';
        $data['orders'] = $this->service_orders->getServiceOrderInvoice($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'status' => 'required',
            ];
            $messages = [
                'status' => [
                    'required' => 'Please select invoice status'
                ]
            ];
            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $data = [
                    's_order_status' => 0,
                ];
                $result = $this->service_orders->update($id, $data);
                if ($result) {
                    $this->session->setTempdata('success', 'Order cancelled successfully', 3);
                    return redirect()->to(site_url('seller/service-orders'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Order could not be cancelled, try again', 3);
                    return redirect()->to(site_url('seller/service-orders'));
                }
            }
        }
        return view('vendors/service-orders/cancel_order', $data);
    }

    // Confirm Invoice
    public function confirm_order($id)
    {
        $data['breadcrumb'] = 'Confirm Invoice';
        $data['orders'] = $this->orders->getOrderInvoice($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'status' => 'required',
            ];
            $messages = [
                'status' => [
                    'required' => 'Please select invoice status'
                ]
            ];
            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $data = [
                    'payment_status' => 1,
                    'order_status' => 1,
                ];
                $result = $this->orders->update($id, $data);
                if ($result) {
                    $this->session->setTempdata('success', 'Order Confirmed successfully', 3);
                    return redirect()->to(site_url('seller/orders'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Order could not be Confirmed, try again', 3);
                    return redirect()->to(site_url('seller/orders'));
                }
            }
        }
        return view('vendors/orders/confirm_order', $data);
    }
    public function confirm_service_order($id)
    {
        $data['breadcrumb'] = 'Confirm Invoice';
        $data['orders'] = $this->service_orders->getServiceOrderInvoice($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'status' => 'required',
            ];
            $messages = [
                'status' => [
                    'required' => 'Please select invoice status'
                ]
            ];
            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                $data = [
                    's_payment_status' => 1,
                    's_order_status' => 1,
                ];
                $result = $this->service_orders->update($id, $data);
                if ($result) {
                    $this->session->setTempdata('success', 'Order Confirmed successfully', 3);
                    return redirect()->to(site_url('seller/service-orders'));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Order could not be Confirmed, try again', 3);
                    return redirect()->to(site_url('seller/service-orders'));
                }
            }
        }
        return view('vendors/service-orders/confirm_order', $data);
    }

    // Finished Orders
    public function finished_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Completed Orders';
        $data['orders'] = $this->orders->getCompletedtOrders($user_id);
        return view('vendors/orders/completed_orders', $data);
    }
    public function completed_service_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Completed Orders';
        $data['orders'] = $this->service_orders->getCompletedServiceOrders($user_id);
        return view('vendors/service-orders/completed_orders', $data);
    }

    // Pending Orders
    public function pending_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Pending Orders';
        $data['orders'] = $this->orders->getPendingOrders($user_id);
        return view('vendors/orders/pending_orders', $data);
    }
    public function pending_service_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Pending Orders';
        $data['orders'] = $this->service_orders->getPendingServiceOrders($user_id);
        return view('vendors/service-orders/pending_orders', $data);
    }

    // Cancelled Orders
    public function cancelled_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Cancelled Orders';
        $data['orders'] = $this->orders->getCancelledOrders($user_id);
        return view('vendors/orders/cancelled_orders', $data);
    }
    public function cancelled_service_orders()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Cancelled Orders';
        $data['orders'] = $this->service_orders->getCancelledServiceOrders($user_id);
        return view('vendors/service-orders/cancelled_orders', $data);
    }
}
