<?php

namespace App\Controllers\Seller;

use App\Controllers\BaseController;
use App\Models\Admin\Categories;
use App\Models\Admin\UnitsModel;
use App\Models\Admin\AttributesModel;
use App\Models\Vendor\ProductsModel;
use App\Models\Vendor\ProductRatingModel;

class Products extends BaseController
{
    public $categories;
    public $units;
    public $attributes;
    public $products;
    public $reviews;
    public $session;
    public $db;

    public function __construct()
    {
        $this->categories = new Categories();
        $this->units = new UnitsModel();
        $this->attributes = new AttributesModel();
        $this->products = new ProductsModel();
        $this->reviews = new ProductRatingModel();
        $this->session = session();
        $this->db = \Config\Database::connect();
    }

    public function index()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Products';
        $data['categories'] = $this->categories->getCategories();
        $data['units'] = $this->units->getUnits();
        $data['products'] = $this->products->getProducts($user_id);
        return view('vendors/products/product_list', $data);
    }
    // Product Reviews
    public function product_reviews()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Reviews';
        $data['reviews'] = $this->reviews->getProductReviews($user_id);
        $data['products'] = $this->products->getProducts($user_id);
        return view('vendors/products/product_reviews', $data);
    }

    public function review_search()
    {

        $user_id = user()->id;
        $data['breadcrumb'] = 'Reviews';
        $product_id = $this->request->getVar('product');
        $data['reviews'] = $this->reviews->getReviewSearch($product_id, $user_id);
        $data['products'] = $this->products->getProducts($user_id);

        return view('vendors/products/review_results', $data);
    }

    // Banned Products
    public function banned_products()
    {
        $user_id = user()->id;
        $data['breadcrumb'] = 'Products';
        $data['categories'] = $this->categories->getCategories();
        $data['units'] = $this->units->getUnits();
        $data['products'] = $this->products->getBannedProducts($user_id);
        return view('vendors/products/banned_product', $data);
    }

    // Add products
    public function add_products()
    {
        $data['breadcrumb'] = 'Add Products';
        $data['categories'] = $this->categories->getCategories();
        $data['units'] = $this->units->getUnits();
        $data['attributes'] = $this->attributes->getAttributes();
        if ($this->request->getMethod() == 'post') {

            $rules = [
                'name' => 'required',
                'keywords' => 'required',
                'meta_desc' => 'required',
                'category'  => 'required',
                'units'  => 'required',
                'sale_type' => 'required',
                'unit_value' => 'required',
                'price' => 'required',
                'description' => 'required',
                'product_image' => 'uploaded[product_image]|max_size[product_image,5054]'
            ];

            $messages = [
                'name' => [
                    'required' => 'Product name is required'
                ],
                'keywords' => [
                    'required' => 'Product keywords is required'
                ],
                'meta_desc' => [
                    'required' => 'Product meta description is required'
                ],
                'sale_type' => [
                    'required' => 'Please choose between wholesale or retail'
                ],
                'unit_value' => [
                    'required' => 'Product unit value is required'
                ],
                'description' => [
                    'required' => 'Product description is required'
                ],
                'product_image' => [
                    'max_size' => 'Product image should not be more 5MBs'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {
                if (!empty($_FILES['product_image']['name'])) {
                    $image = $this->request->getFile('product_image');
                    $new_image = $image->getRandomName();
                }

                $product_ref = random_string('alnum', 10);
                $sku = "AG" . random_string('alnum', 5);
                $data = [
                    'product_ref' => $product_ref,
                    'product_category' => $this->request->getVar('category'),
                    'product_vendor' => $this->request->getVar('vendor'),
                    'product_name' => ucwords($this->request->getVar('name')),
                    'product_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'product_keywords' => $this->request->getVar('keywords'),
                    'product_meta_desc' => $this->request->getVar('meta_desc'),
                    'product_desc' => $this->request->getVar('description'),
                    'product_image' => $new_image,
                    'product_unit' => $this->request->getVar('units'),
                    'sale_type' => $this->request->getVar('sale_type'),
                    'unit_value' => $this->request->getVar('unit_value'),
                    'sku' => $sku,
                    'product_price' => $this->request->getVar('price'),
                    'product_status' => 1,
                    'product_district' => $this->request->getVar('district'),
                    'product_market' => $this->request->getVar('market'),
                ];

                $result = $this->products->save($data);

                if ($result) {
                    if (!empty($_FILES['product_image']['name'])) {
                        $image->move('public/uploads/products/products', $new_image);
                    }
                    $this->session->setTempdata('success', 'Product added successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Product could not be added, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('vendors/products/add_products', $data);
    }

    // Edit Products
    public function edit_product($id)
    {
        $data['breadcrumb'] = 'Edit Product';
        $data['categories'] = $this->categories->getCategories();
        $data['units'] = $this->units->getUnits();
        $data['product'] = $this->products->getProductID($id);
        if ($this->request->getMethod() == 'post') {
            $rules = [
                'name' => 'required',
                'keywords' => 'required',
                'meta_desc' => 'required',
                'category'  => 'required',
                'units'  => 'required',
                'unit_value' => 'required',
                'price' => 'required',
                'description' => 'required',
            ];

            $messages = [
                'name' => [
                    'required' => 'Product name is required'
                ],
                'keywords' => [
                    'required' => 'Product keywords is required'
                ],
                'meta_desc' => [
                    'required' => 'Product meta description is required'
                ],
                'unit_value' => [
                    'required' => 'Product unit value is required'
                ],
                'description' => [
                    'required' => 'Product description is required'
                ],
            ];

            if (!$this->validate($rules, $messages)) {
                $data['validation'] = $this->validator;
            } else {

                if (!empty($_FILES['product_image']['name'])) {
                    // Unlink old image
                    $findFile = $this->products->getProductID($id);
                    $old_img = $findFile->product_image;
                    $imagePath = 'public/uploads/products/products/' . $old_img;

                    if (file_exists($imagePath)) {
                        unlink('public/uploads/products/products/' . $old_img);
                    }

                    $image = $this->request->getFile('product_image');
                    $new_image = $image->getRandomName();
                }

                $data = [
                    'product_category' => $this->request->getVar('category'),
                    'product_vendor' => $this->request->getVar('vendor'),
                    'product_name' => ucwords($this->request->getVar('name')),
                    'product_slug' => url_title(strtolower($this->request->getVar('name')), '-', true),
                    'product_desc' => $this->request->getVar('description'),
                    'product_keywords' => $this->request->getVar('keywords'),
                    'product_meta_desc' => ucfirst($this->request->getVar('meta_desc')),
                    'product_image' => (!empty($_FILES['product_image']['name']) ? $new_image : $this->request->getVar('old_image')),
                    'product_unit' => $this->request->getVar('units'),
                    'unit_value' => $this->request->getVar('unit_value'),
                    'product_price' => $this->request->getVar('price'),
                ];

                $result = $this->products->update($id, $data);

                if ($result) {

                    if (!empty($_FILES['product_image']['name'])) {
                        $image->move('public/uploads/products/products', $new_image);
                    }
                    $this->session->setTempdata('success', 'Product updated successfully', 3);
                    return redirect()->to(current_url());
                } else {
                    $this->session->setTempdata('error', 'Sorry! Product could not be updated, try again', 3);
                    return redirect()->to(current_url());
                }
            }
        }
        return view('vendors/products/edit_product', $data);
    }

    // Delete product
    public function delete_product($id)
    {
        // Unlink old image
        $findFile = $this->products->getProductID($id);
        $old_img = $findFile->product_image;
        $imagePath = 'public/uploads/products/products/' . $old_img;

        if (file_exists($imagePath)) {
            unlink('public/uploads/products/products/' . $old_img);
        }

        $result = $this->products->delete($id);
        if ($result) {
            $this->session->setTempdata('success', 'Product Deleted successfully', 3);
            return redirect()->to(site_url('seller/products'));
        }
    }
}
