<?php

namespace App\Controllers\Web;

use App\Controllers\BaseController;
use App\Models\Web\MarketsModel;
use App\Models\Web\DistrictsModel;
use App\Models\Web\ServicesModel;
use App\Models\Web\UserRequestsModel;
use App\Models\Web\ReportAbuseModel;
use App\Models\Web\ProductsModel;
use App\Models\Web\ProductReviewsModel;
use App\Models\Web\ProductOrdersModel;
use App\Models\Web\ServiceOrdersModel;
use App\Models\Web\ServiceReviewsModel;
use App\Models\Web\VendorsModel;
use App\Models\Web\HomeModel;
use App\Models\Web\PriceWatchModel;
use App\Models\Web\UserRegistrationModel;

class Marketplace extends BaseController
{
    public $markets;
    public $districts;
    public $services;
    public $requests;
    public $report;
    public $auth;
    public $product;
    public $review;
    public $product_orders;
    public $service_reviews;
    public $service_orders;
    public $vendors;
    public $home;
    public $price_watch;
    public $register;
    public $pager;
    public $db;
    public $config;
    public $session;

    public function __construct()
    {
        $this->markets = new MarketsModel();
        $this->districts = new DistrictsModel();
        $this->services = new ServicesModel();
        $this->report = new ReportAbuseModel();
        $this->requests = new UserRequestsModel();
        $this->product = new ProductsModel();
        $this->review = new ProductReviewsModel();
        $this->product_orders = new ProductOrdersModel();
        $this->service_orders = new ServiceOrdersModel();
        $this->service_reviews = new ServiceReviewsModel();
        $this->vendors = new VendorsModel();
        $this->home = new HomeModel();
        $this->price_watch = new PriceWatchModel();
        $this->register = new UserRegistrationModel();
        $this->pager = \Config\Services::pager();
        $this->db = \Config\Database::connect();
        $this->config = config('Auth');
        $this->auth = service('authentication');
        $this->session = session();
        $pager = \Config\Services::pager();
    }

    // Popular markets lists
    public function index()
    {
        $data['page_title'] = 'Products';
        $data['all_districts'] = $this->districts->getAllDistricts();
        $data['districts'] = $this->districts->getPopularDistricts();
        $data['markets'] = $this->markets->getPopularMarkets();
        $data['sellers'] = $this->vendors->getRecentVendors();
        $data['db'] = $this->db;
        return view('web/marketplace/products', $data);
    }

    // Buyer Requests
    public function buyer_request()
    {
        $requests = new \App\Models\Web\BuyerRequestsModel();
        $data = [
            'page_title' => "Buyer Requests",
            'requests' => $requests->orderBy('request_id', 'DESC')->paginate(5),
            'pager' => $requests->pager,
        ];

        return view('web/marketplace/buyer_requests', $data);
    }

    // Register New User
    public function user_register()
    {

        $data = [
            'page_title' => 'Register',
            'groups' => $this->db->query("SELECT * FROM auth_groups WHERE name!=" . $this->db->escape('Admin') . " OR name!=" . $this->db->escape('admin') . "")->getResult(),
            'districts' => $this->db->query("SELECT * FROM districts ORDER BY district_name ASC")->getResult(),

        ];

        if ($this->request->getMethod() == 'post') {

            $category = $this->request->getVar('category');

            // If user type is Buyer
            if ($category == 3) {

                $rules = [
                    'firstname' => 'required',
                    'lastname' => 'required',
                    'phone' => 'required|min_length[10]|is_unique[users.phone]',
                    'district' => 'required',
                    'address' => 'required',
                    'password' => 'required|min_length[8]',
                    'pass_confirm' => 'required|matches[password]',
                ];

                $message = [
                    'firstname' => [
                        'required' => 'First name is required',
                    ],
                    'lastname' => [
                        'required' => 'Last name is required',
                    ],
                    'phone' => [
                        'required' => 'Phone is required',
                        'is_unique' => 'Phone already exists',
                    ],
                    'district' => [
                        'required' => 'District is required',
                    ],
                    'address' => [
                        'required' => 'Address is required',
                    ],
                    'password' => [
                        'required' => 'Password is required',
                        'min_length' => 'Password should be 8 characters of more',
                    ],
                    'pass_confirm' => [
                        'required' => 'Confirm Password is required',
                        'matches' => 'The two Passwords donot match',
                    ],
                ];

                if (!$this->validate($rules, $message)) {
                    $data['validation'] = $this->validator;
                } else {

                    $config = config('Auth');
                    if (
                        (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                        ||
                        (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
                    ) {
                        $hashOptions = [
                            'memory_cost' => $config->hashMemoryCost,
                            'time_cost'   => $config->hashTimeCost,
                            'threads'     => $config->hashThreads,
                        ];
                    } else {
                        $hashOptions = [
                            'cost' => $config->hashCost,
                        ];
                    }

                    $password = password_hash(self::preparePassword($this->request->getVar('password')), $this->config->hashAlgorithm, $hashOptions);

                    $data = [
                        'firstname' => ucwords($this->request->getVar('firstname')),
                        'lastname' => ucwords($this->request->getVar('lastname')),
                        'phone' => $this->request->getVar('phone'),
                        'category' => $category,
                        'address' => $this->request->getVar('address'),
                        'district' => $this->request->getVar('district'),
                        'username' => random_string('alnum', 8),
                        'password_hash' => $password,
                        'active' => 1,
                        'verification' => 1
                    ];

                    $result = $this->register->save($data);

                    if ($result) {
                        $userid = $this->register->getInsertID();
                        $this->db->query("INSERT INTO auth_groups_users(group_id, user_id) VALUES(" . $this->db->escape($category) . ", " . $this->db->escape($userid) . ")");
                        $this->session->setTempdata('success', 'Account created successfully', 3);
                        return redirect()->to(current_url());
                    }
                }
            } elseif ($category == 2) {

                $rules = [
                    'firstname' => 'required',
                    'lastname' => 'required',
                    'phone' => 'required|min_length[10]|is_unique[users.phone]',
                    'district' => 'required',
                    'address' => 'required',
                    'market' => 'required',
                    'store_name' => 'required|is_unique[users.store_name]',
                    'sellas' => 'required',
                    'password' => 'required|min_length[8]',
                    'pass_confirm' => 'required|matches[password]',
                ];

                $message = [
                    'firstname' => [
                        'required' => 'First name is required',
                    ],
                    'lastname' => [
                        'required' => 'Last name is required',
                    ],
                    'phone' => [
                        'required' => 'Phone is required',
                        'is_unique' => 'Phone already exists',
                    ],
                    'district' => [
                        'required' => 'District is required',
                    ],
                    'address' => [
                        'required' => 'Address is required',
                    ],
                    'sellas' => [
                        'required' => 'Please select if your selling as an individual or Farmer Group',
                    ],
                    'market' => [
                        'required' => 'Please select your market',
                    ],
                    'store_name' => [
                        'required' => 'Your store name is required',
                        'is_unique' => 'Store name already exists',
                    ],
                    'password' => [
                        'required' => 'Password is required',
                        'min_length' => 'Password should be 8 characters of more',
                    ],
                    'pass_confirm' => [
                        'required' => 'Confirm Password is required',
                        'matches' => 'The two Passwords donot match',
                    ],
                ];

                if (!$this->validate($rules, $message)) {
                    $data['validation'] = $this->validator;
                } else {

                    $config = config('Auth');
                    if (
                        (defined('PASSWORD_ARGON2I') && $config->hashAlgorithm == PASSWORD_ARGON2I)
                        ||
                        (defined('PASSWORD_ARGON2ID') && $config->hashAlgorithm == PASSWORD_ARGON2ID)
                    ) {
                        $hashOptions = [
                            'memory_cost' => $config->hashMemoryCost,
                            'time_cost'   => $config->hashTimeCost,
                            'threads'     => $config->hashThreads,
                        ];
                    } else {
                        $hashOptions = [
                            'cost' => $config->hashCost,
                        ];
                    }

                    $password = password_hash(self::preparePassword($this->request->getVar('password')), $this->config->hashAlgorithm, $hashOptions);

                    $data = [
                        'firstname' => ucwords($this->request->getVar('firstname')),
                        'lastname' => ucwords($this->request->getVar('lastname')),
                        'store_name' => ucwords($this->request->getVar('store_name')),
                        'slug' => url_title($this->request->getVar('store_name'), '-', true),
                        'phone' => $this->request->getVar('phone'),
                        'category' => $category,
                        'address' => $this->request->getVar('address'),
                        'district' => $this->request->getVar('district'),
                        'market' => $this->request->getVar('market'),
                        'username' => url_title($this->request->getVar('store_name'), '_', true),
                        'password_hash' => $password,
                        'active' => 1,
                        'verification' => 1
                    ];

                    $result = $this->register->save($data);

                    if ($result) {
                        $userid = $this->register->getInsertID();
                        $this->db->query("INSERT INTO auth_groups_users(group_id, user_id) VALUES(" . $this->db->escape($category) . ", " . $this->db->escape($userid) . ")");
                        $this->session->setTempdata('success', 'Account created successfully', 3);
                        return redirect()->to(current_url());
                    }
                }
            } else {
                $this->session->setTempdata('error', 'Sorry! Unknown User Type', 3);
                return redirect()->to(current_url());
            }
        }

        return view('web/register', $data);
    }

    protected static function preparePassword(string $password): string
    {
        return base64_encode(hash('sha384', $password, true));
    }

    // Get District Markets
    public function get_district_markets()
    {

        if (isset($_POST["dist"])) {
            $district = $_POST["dist"];
            $query = "SELECT * FROM markets WHERE market_district=" . $district . "";
        }

        $statement = $this->db->query($query);
        $result = $statement->getResult();
        $output = '';

        if ($result) {
            $output .= "
            <div class='form-group'>
                <label for='district'>Market</label>
                <select class='custom-select' name='market' id='mkt'>
                    <option value='' selected>Select Market</option>";
            foreach ($result as $row) :
                $output .= "<option value=" . $row->market_id . ">" . $row->market_name . "</option>";
            endforeach;
            $output .=  "
                </select>
            </div>";
        } else {
            $output .= "
            <div class='form-group'>
                <label for='district'>Market</label>
                <select class='custom-select' name='market' id='mkt'>
                    <option value='' selected>Select Market</option>
                </select>
            </div>";
        }

        echo $output;
    }

    public function district_ajax()
    {

        if (isset($_POST["action"])) {

            $response = '<ul><li>No data found!</li></ul>';

            $query = "SELECT * FROM districts";

            if (isset($_POST["district"])) {
                $districts_filter = $_POST["district"];
                $query = "SELECT * FROM districts WHERE district_name LIKE '%$districts_filter%'";
            }

            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                $response = '<ul>';
                foreach ($result as $row) {
                    $response .= '<li><a href="' . site_url("district/" . $row->district_slug) . '">' . $row->district_name . '</a></li>';
                }
                $response .= '</ul>';
            }

            exit($response);
        } else {
            $response = '<ul><li>No data found!</li></ul>';
            $query = "SELECT * FROM districts";
            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                $response = '<ul>';
                foreach ($result as $row) {
                    $response .= '<li><a href="' . site_url("district/" . $row->district_slug) . '">' . $row->district_name . '</a></li>';
                }
                $response .= '</ul>';
            }

            exit($response);
        }
    }

    // Service district ajax
    public function service_district_ajax()
    {

        if (isset($_POST["action"])) {

            $response = '<ul><li>No data found!</li></ul>';

            $query = "SELECT * FROM districts";

            if (isset($_POST["district"])) {
                $districts_filter = $_POST["district"];
                $query = "SELECT * FROM districts WHERE district_name LIKE '%$districts_filter%'";
            }

            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                $response = '<ul>';
                foreach ($result as $row) {
                    $response .= '<li><a href="' . site_url("services/districts/" . $row->district_slug) . '">' . $row->district_name . '</a></li>';
                }
                $response .= '</ul>';
            }

            exit($response);
        } else {
            $response = '<ul><li>No data found!</li></ul>';
            $query = "SELECT * FROM districts";
            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                $response = '<ul>';
                foreach ($result as $row) {
                    $response .= '<li><a href="' . site_url("services/districts/" . $row->district_slug) . '">' . $row->district_name . '</a></li>';
                }
                $response .= '</ul>';
            }

            exit($response);
        }
    }

    public function category_ajax()
    {

        if (isset($_POST["action"])) {

            $response = '<ul><li>No data found!</li></ul>';

            $query = "SELECT * FROM service_types";

            if (isset($_POST["type"])) {
                $service_filter = $_POST["type"];
                $query = "SELECT * FROM service_types WHERE service_type_name LIKE '%$service_filter%'";
            }

            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                $response = '<ul>';
                foreach ($result as $row) {
                    $response .= '<li><a href="' . site_url("services-by-category/" . $row->service_type_slug) . '">' . $row->service_type_name . '</a></li>';
                }
                $response .= '</ul>';
            }

            exit($response);
        } else {
            $response = '<ul><li>No data found!</li></ul>';
            $query = "SELECT * FROM service_types";
            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                $response = '<ul>';
                foreach ($result as $row) {
                    $response .= '<li><a href="' . site_url("services-by-category/" . $row->service_type_slug) . '">' . $row->service_type_name . '</a></li>';
                }
                $response .= '</ul>';
            }

            exit($response);
        }
    }

    // Product marketplace
    public function product_market()
    {
        $data['page_title'] = "Products";
        $data['all_districts'] = $this->districts->getAllDistricts();
        $data['recents'] = $this->product->getRecentProducts();
        $data['markets'] = $this->markets->getAllMarkets();
        $data['categories'] = $this->product->getCategories();
        $data['sellers'] = $this->vendors->getRecentVendors();

        return view('web/marketplace/product_market', $data);
    }

    public function filter_products()
    {

        if (isset($_POST["action"])) {

            $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1";

            if (isset($_POST["district_data"])) {
                $districts_filter = implode("','", $_POST["district_data"]);
                $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1 AND product_district IN('" . $districts_filter . "')";
            }

            if (isset($_POST["market_data"])) {
                $markets_filter = implode("','", $_POST["market_data"]);
                $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1 AND product_market IN('" . $markets_filter . "')";
            }

            if (isset($_POST["category_data"])) {
                $categories_filter = implode("','", $_POST["category_data"]);
                $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1 AND product_category IN('" . $categories_filter . "')";
            }

            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                foreach ($result as $row) {
                    if ($row->verification == 1) {
                        $output .= '
                    <div class="col-lg-4 col-md-4 col-sm-6 col-12">
                        <div class="card products">
                            <a href="' . site_url("products/" . $row->product_slug) . '">
                            <div class="card-body">
                                <div class="product-image">
                                    <img src="' . base_url('public/uploads/products/products/' . $row->product_image) . '" alt="Product Image" class="img-fluid">
                                </div>
                                <div class="product-category">
                                    <span>' . $row->category_name . '</span>
                                </div>
                            </div>
                            </a>
                            <div class="card-footer">
                                <div class="seller">
                                    <span><b>Seller:</b> ' . $row->store_name . '</span>
                                </div>
                                <h6><a href="' . site_url("products/" . $row->product_slug) . '">' . substr($row->product_name, 0, 15) . "..." . '</a></h6>
                                <span class="stock">IN STOCK</span>
                                <a href="' . site_url("products/" . $row->product_slug) . '" class="message">Details</a>
                                <div class="product-price">
                                    <h5>' . number_format($row->product_price) . '-' .  $row->unit_shortname . '</h5>
                                </div>
                            </div>
                        </div>
                    </div>
                    ';
                    }
                }
            } else {
                $output = '<h3>No Data Found</h3>';
            }
            echo $output;
        }
    }

    // Market details
    public function market_products($slug = null)
    {
        $market = $this->markets->getMarketId($slug);
        $data['page_title'] = $market->market_name;
        $data['all_districts'] = $this->districts->getAllDistricts();
        $data['products'] = $this->product->getProductsByMarket($market->market_id);
        $data['recents'] = $this->product->getRecentProducts();
        $data['markets'] = $this->markets->getAllMarkets();
        $data['categories'] = $this->product->getCategories();
        $data['db'] = $this->db;
        return view('web/marketplace/products_by_market', $data);
    }

    public function filter_market()
    {

        if (isset($_POST["action"])) {

            $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1 ORDER BY product_id DESC";

            if (isset($_POST["category_data"])) {
                $categories_filter = implode("','", $_POST["category_data"]);
                $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1 AND product_category IN('" . $categories_filter . "')";
            }

            if (isset($_POST["market_data"])) {
                $markets_filter = implode("','", $_POST["market_data"]);
                $query = "SELECT * FROM products as p INNER JOIN categories as c ON c.category_id=p.product_category INNER JOIN users as u ON u.id=p.product_vendor INNER JOIN units as un ON un.unit_id=p.product_unit WHERE product_status = 1 AND product_market IN('" . $markets_filter . "')";
            }

            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                foreach ($result as $row) {
                    if ($row->verification == 1) {
                        $output .= '
                    <div class="col-lg-4 col-md-4 col-sm-6 col-12">
                        <div class="card products">
                            <a href="' . site_url("products/" . $row->product_slug) . '">
                            <div class="card-body">
                                <div class="product-image">
                                    <img src="' . base_url('public/uploads/products/products/' . $row->product_image) . '" alt="Product Image" class="img-fluid">
                                </div>
                                <div class="product-category">
                                    <span>' . $row->category_name . '</span>
                                </div>
                            </div>
                            </a>
                            <div class="card-footer">
                                <div class="seller">
                                    <span><b>Seller:</b> ' . $row->store_name . '</span>
                                </div>
                                <h6><a href="' . site_url("products/" . $row->product_slug) . '">' . $row->product_name . '</a></h6>
                                <span class="stock">IN STOCK</span>
                                <a href="' . site_url("products/" . $row->product_slug) . '" class="message">Details</a>
                                <div class="product-price">
                                    <h5>' . number_format($row->product_price) . '-' .  $row->unit_shortname . '</h5>
                                </div>
                            </div>
                        </div>
                    </div>
                    ';
                    }
                }
            } else {
                $output = '<h3>No Data Found</h3>';
            }
            echo $output;
        }
    }

    public function district_markets($slug = null)
    {
        $slug = $this->districts->getDistrictId($slug);
        $data['page_title'] = $slug->district_name;
        $data['markets'] = $this->product->getMarketsByDistrict($slug->district_id);
        $data['all_districts'] = $this->districts->getAllDistricts();
        $data['sellers'] = $this->vendors->getRecentVendors();
        $data['db'] = $this->db;
        return view('web/marketplace/district_markets', $data);
    }

    public function market_filter()
    {
        return view('web/market_filter');
    }

    public function service_marketplace()
    {
        // $data['services'] = $this->services->getAllServices();
        $data['districts'] = $this->districts->getAllDistricts();
        $data['categories'] = $this->services->getAllServiceCategories();
        $data['recent_services'] = $this->services->getRecentServices();
        $data['sellers'] = $this->vendors->getRecentVendors();
        $data['db'] = $this->db;
        return view('web/services/service_marketplace', $data);
    }

    public function filter_services()
    {
        if (isset($_POST["action"])) {

            $query = "SELECT * FROM services as s INNER JOIN service_types as t ON t.service_type_id=s.service_category INNER JOIN users as u ON u.id=s.service_vendor INNER JOIN service_units as un ON un.service_unit_id=s.service_unit WHERE service_status = 1";

            if (isset($_POST["district_data"])) {
                $districts_filter = implode("','", $_POST["district_data"]);
                $query = "SELECT * FROM services as s INNER JOIN service_types as t ON t.service_type_id=s.service_category INNER JOIN users as u ON u.id=s.service_vendor INNER JOIN service_units as un ON un.service_unit_id=s.service_unit WHERE service_status = 1 AND service_district IN('" . $districts_filter . "')";
            }

            if (isset($_POST["category_data"])) {
                $categories_filter = implode("','", $_POST["category_data"]);
                $query = "SELECT * FROM services as s INNER JOIN service_types as t ON t.service_type_id=s.service_category INNER JOIN users as u ON u.id=s.service_vendor INNER JOIN service_units as un ON un.service_unit_id=s.service_unit WHERE service_status = 1 AND service_category IN('" . $categories_filter . "')";
            }

            $statement = $this->db->query($query);
            $result = $statement->getResult();
            $output = '';
            if ($result) {
                foreach ($result as $row) {
                    if ($row->verification == 1) {
                        $output .= '
                    <div class="col-lg-4 col-md-4 col-sm-6 col-12">
                        <div class="card service-card">
                            <a href="' . site_url('services/' . esc($row->service_slug)) . '">
                            <div class="card-body">
                                <div class="service-image">
                                    <img src="' . base_url('public/uploads/services/' . $row->service_image) . '" alt="Service" class="img-fluid">
                                </div>
                                <div class="profile-img">';
                        if (!empty($row->profile_image)) :
                            $output .= ' <img src="' . base_url('public/uploads/users/profiles/' . $row->profile_image) . '" alt="Profile" class="img-fluid">';
                        else :
                            $output .= '<img src="' . base_url('public/uploads/defaults/user.png/') . '" alt="Profile" class="img-fluid">';
                        endif;
                        $output .= '</div>
                            </div>
                            </a>
                            <div class="card-footer">
                                <div class="badges">
                                    <span class="badge verified">Verified</span>
                                    <span class="badge badge-primary price">' . number_format($row->service_price) . ' / ' . $row->service_unit_name . '</span>
                                </div>
                                <p class="service-type">' . $row->service_type_name . '</p>
                                <h6><a href="' . site_url('services/' . esc($row->service_slug)) . '">' . $row->service_name . '</a></h6>
                                <p class="provider">By: <a href="' . site_url('sellers/' . $row->slug) . '">' . $row->store_name . '</a></p>
                                <hr>
                                <p class="address">' . $row->address . '</p>
                            </div>
                        </div>
                    </div>
                    ';
                    }
                }
            } else {
                $output = '<h4 class="text-center py-5" style="text-align: center !important;">No Data Found</h4>';
            }
            echo $output;
        }
    }

    public function services()
    {
        $data['page_title'] = "Services";
        $data['all_districts'] = $this->districts->getAllDistricts();
        $data['districts'] = $this->services->getRandomDistricts();
        $data['popular_categories'] = $this->services->getPopularCategories();
        $data['recent_services'] = $this->services->getRecentServices();
        $data['sellers'] = $this->vendors->getRecentVendors();
        $data['db'] = $this->db;
        return view('web/services/services', $data);
    }

    public function service_by_district($slug)
    {
        $data['all_districts'] = $this->districts->getAllDistricts();
        $slug = $this->services->getFilterDistricts($slug);
        $data['page_title'] = $slug->district_name;
        // Get vendors in a district
        $userResult = $this->db->query("SELECT * FROM users WHERE category=2 AND verification=1 AND district='$slug->district_id'");
        $data['users'] = $userResult->getResult();
        $data['featured'] = $this->vendors->getFeaturedVendors();
        $data['sellers'] = $this->vendors->getRecentVendors();
        $data['db'] = $this->db;
        return view('web/services/district_details', $data);
    }

    // Product Details
    public function product_details($slug = null)
    {
        $data['auth'] = $this->auth;
        $product = $this->product->getProductDetails($slug);
        $data['page_title'] = $product->product_name;
        $data['product'] = $this->product->getProductDetails($slug);
        $data['reviews'] = $this->product->getProductReviews($product->product_id);
        $data['countReview'] = $this->product->countReviews($product->product_id);
        $data['related_products'] = $this->product->getRelatedProduct($product->product_category);
        return view('web/marketplace/product_details', $data);
    }

    // Review product
    public function review_product($id = null)
    {

        if ($this->request->getMethod() == 'post') {

            $slug = $this->request->getVar('slug');

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('products/' . $slug));
            }

            $rules = [
                'rating' => 'required|max_length[3]',
                'title' => 'required|',
                'description' => 'required|max_length[500]'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'reviewer_id' => $this->request->getVar('customer'),
                    'r_product_vendor' => $this->request->getVar('vendor'),
                    'reviewed_product' => $this->request->getVar('product'),
                    'review_title' => ucwords($this->request->getVar('title')),
                    'review_desc' => $this->request->getVar('description'),
                    'review_rating' => $this->request->getVar('rating'),
                ];

                $result = $this->review->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Review submitted successfully', 3);
                    return redirect()->to(site_url('products/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Failed to submit Review', 3);
                    return redirect()->to(site_url('products/' . $slug));
                }
            }
        }
    }

    // Review Service
    public function review_service($id = null)
    {

        if ($this->request->getMethod() == 'post') {

            $slug = $this->request->getVar('slug');

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('services/' . $slug));
            }

            $rules = [
                'rating' => 'required|max_length[3]',
                'title' => 'required',
                'description' => 'required|max_length[500]'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {

                $data = [
                    'reviewer_id' => $this->request->getVar('customer'),
                    'r_service_vendor' => $this->request->getVar('vendor'),
                    'reviewed_service' => $this->request->getVar('service'),
                    'review_title' => ucwords($this->request->getVar('title')),
                    'review_desc' => $this->request->getVar('description'),
                    'review_rating' => $this->request->getVar('rating'),
                ];

                $result = $this->service_reviews->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Review submitted successfully', 3);
                    return redirect()->to(site_url('services/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Failed to submit Review', 3);
                    return redirect()->to(site_url('services/' . $slug));
                }
            }
        }
    }

    // Service Details
    public function service_details($slug = null)
    {
        $data['auth'] = $this->auth;
        $services = $this->services->getServiceDetails($slug);
        $data['page_title'] = $services->service_name;
        $data['services'] = $this->services->getServiceDetails($slug);
        $data['reviews'] = $this->services->getServiceReviews($services->service_id);
        $data['countReview'] = $this->services->countReviews($services->service_id);
        $data['related_services'] = $this->services->getRelatedServices($services->service_category);
        return view('web/marketplace/service_details', $data);
    }

    // Send requests
    public function send_request($slug = null)
    {

        if ($this->request->getMethod() == 'post') {

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('sellers/' . $slug));
            }

            $rules = [
                'subject' => 'required|max_length[150]',
                'type' => 'required',
                'description' => 'required|max_length[255]'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {

                $request_ref = 'REF' . random_string('alnum', 7);

                $data = [
                    'request_ref' => $request_ref,
                    'user_id' => $this->request->getVar('user_id'),
                    'vendor_id' => $this->request->getVar('vendor_id'),
                    'request_title' => $this->request->getVar('subject'),
                    'request_type' => $this->request->getVar('type'),
                    'request_desc' => $this->request->getVar('description'),
                    'request_status' => 2,
                    'read_status' => 0,
                ];

                $result = $this->requests->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Request submitted successfully', 3);
                    return redirect()->to(site_url('sellers/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Failed to submit request', 3);
                    return redirect()->to(site_url('sellers/' . $slug));
                }
            }
        }
    }

    // Customer request
    public function custom_request()
    {

        if ($this->request->getMethod() == 'post') {

            $slug = $this->request->getVar('product_slug');

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('products/' . $slug));
            }

            $rules = [
                'subject' => 'required|max_length[150]',
                'type' => 'required',
                'description' => 'required|max_length[255]'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {

                $request_ref = 'REF' . random_string('alnum', 7);

                $data = [
                    'request_ref' => $request_ref,
                    'user_id' => $this->request->getVar('user_id'),
                    'vendor_id' => $this->request->getVar('vendor_id'),
                    'request_title' => $this->request->getVar('subject'),
                    'request_type' => $this->request->getVar('type'),
                    'request_desc' => $this->request->getVar('description'),
                    'request_status' => 2,
                    'read_status' => 0,
                ];

                $result = $this->requests->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Request submitted successfully', 3);
                    return redirect()->to(site_url('products/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Failed to submit request', 3);
                    return redirect()->to(site_url('products/' . $slug));
                }
            }
        }
    }

    // Service request
    public function service_request()
    {

        if ($this->request->getMethod() == 'post') {

            $slug = $this->request->getVar('service_slug');

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('services/' . $slug));
            }

            $rules = [
                'subject' => 'required|max_length[150]',
                'type' => 'required',
                'description' => 'required|max_length[255]'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {

                $request_ref = 'REF' . random_string('alnum', 7);

                $data = [
                    'request_ref' => $request_ref,
                    'user_id' => $this->request->getVar('user_id'),
                    'vendor_id' => $this->request->getVar('vendor_id'),
                    'request_title' => $this->request->getVar('subject'),
                    'request_type' => $this->request->getVar('type'),
                    'request_desc' => $this->request->getVar('description'),
                    'request_status' => 2,
                    'read_status' => 0,
                ];

                $result = $this->requests->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Request submitted successfully', 3);
                    return redirect()->to(site_url('services/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Failed to submit request', 3);
                    return redirect()->to(site_url('services/' . $slug));
                }
            }
        }
    }

    // Report Vendor
    public function report_vendor()
    {

        if ($this->request->getMethod() == 'post') {

            $slug = $this->request->getVar('vender_slug');

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('sellers/' . $slug));
            }

            $rules = [
                'reason' => 'required',
                'description' => 'required|max_length[255]'
            ];

            if (!$this->validate($rules)) {
                $data['validation'] = $this->validator;
            } else {

                $report_ref = 'REF' . random_string('alnum', 7);

                $data = [
                    'report_ref' => $report_ref,
                    'reason' => $this->request->getVar('reason'),
                    'user_id' => $this->request->getVar('user_id'),
                    'vendor_id' => $this->request->getVar('vendor_id'),
                    'description' => $this->request->getVar('description'),
                    'read_status' => 0,
                ];

                $result = $this->report->save($data);

                if ($result) {
                    $this->session->setTempdata('success', 'Complaint submitted successfully', 3);
                    return redirect()->to(site_url('sellers/' . $slug));
                } else {
                    $this->session->setTempdata('error', 'Sorry! Failed to submit Complaint', 3);
                    return redirect()->to(site_url('sellers/' . $slug));
                }
            }
        }
    }

    // Place order
    public function place_order($slug)
    {

        if ($this->request->getMethod() == 'post') {

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('products/' . $slug));
            }

            $inv_ref = 'INV' . random_string('alnum', 7);
            $qty = $this->request->getVar('qty');
            $price = $this->request->getVar('price');
            $total = $qty * $price;

            $data = [
                'order_invoice' => $inv_ref,
                'user_id' => $this->request->getVar('user_id'),
                'vendor_id' => $this->request->getVar('vendor_id'),
                'prod_id' => $this->request->getVar('prod_id'),
                'pro_order_unit' => $this->request->getVar('order_unit'),
                'prod_district' => $this->request->getVar('product_district'),
                'prod_market' => $this->request->getVar('product_market'),
                'prod_category' => $this->request->getVar('product_category'),
                'order_qty' => $qty,
                'order_total' => $total,
                'order_discount' => 0,
                'payment_status' => 0,
                'order_status' => 2,
                'read_status' => 0,
            ];

            $result = $this->product_orders->save($data);

            if ($result) {
                $this->session->setTempdata('success', 'Order placed successfully', 3);
                return redirect()->to(site_url('products/' . $slug));
            } else {
                $this->session->setTempdata('error', 'Sorry! Failed to place Order', 3);
                return redirect()->to(site_url('products/' . $slug));
            }
        }
    }

    // Book service
    public function place_service_order($slug)
    {

        if ($this->request->getMethod() == 'post') {

            if (!$this->auth->check()) {
                $this->session->setTempdata('error', 'Please login first', 3);
                return redirect()->to(site_url('services/' . $slug));
            }

            $inv_ref = 'INV' . random_string('alnum', 7);
            $qty = $this->request->getVar('qty');
            $price = $this->request->getVar('price');
            $total = $qty * $price;

            $data = [
                's_order_invoice' => $inv_ref,
                's_user_id' => $this->request->getVar('user_id'),
                's_vendor_id' => $this->request->getVar('vendor_id'),
                's_service_id' => $this->request->getVar('service_id'),
                's_service_unit' => $this->request->getVar('service_unit'),
                's_service_unit_price' => $this->request->getVar('price'),
                's_category' => $this->request->getVar('s_category'),
                's_district' => $this->request->getVar('s_district'),
                's_order_qty' => $qty,
                's_order_total' => $total,
                's_order_discount' => 0,
                's_payment_status' => 0,
                's_order_status' => 2,
                'read_status' => 0,
            ];

            $result = $this->service_orders->save($data);

            if ($result) {
                $this->session->setTempdata('success', 'Service booked successfully', 3);
                return redirect()->to(site_url('services/' . $slug));
            } else {
                $this->session->setTempdata('error', 'Sorry! Failed to booked service', 3);
                return redirect()->to(site_url('services/' . $slug));
            }
        }
    }

    // Seller Search
    public function seller_search($slug = null)
    {
        $data['all_districts'] = $this->districts->getAllDistricts();
        $query = $this->db->query("SELECT * FROM districts WHERE district_slug='$slug'");
        $result = $query->getRow();
        $user_query = $this->db->query("SELECT * FROM users WHERE district='$result->district_id' AND category=2 AND verification=1");
        $data['page_title'] = $result->district_name;
        $data['district'] = $result->district_name;
        $data['count'] = $this->services->countServiceResults($result->district_id);
        $data['sellers'] = $user_query->getResult();
        return view('web/vendors/sellers_by_district', $data);
    }

    // Search services by categories
    public function services_by_category($slug = null)
    {
        $query = $this->db->query("SELECT * FROM service_types WHERE service_type_slug='$slug'");
        $result = $query->getRow();
        $data['page_title'] = $result->service_type_name;
        $data['categories'] = $this->services->getAllServiceCategories();
        $data['districts'] = $this->districts->getAllDistricts();
        $data['services'] = $this->services->getServicesByCategory($result->service_type_id);
        $data['sellers'] = $this->vendors->getRecentVendors();
        $data['count'] = $this->services->countServiceByCategory($result->service_type_id);
        $data['db'] = $this->db;
        return view('web/services/services_by_category', $data);
    }

    public function filter()
    {
        return view('web/filter');
    }

    public function markets()
    {
        return view('web/markets');
    }

    public function price_watch()
    {
        $data['page_title'] = 'Market Analytics';
        $data['product_watch'] = $this->price_watch->market_highlights();
        $data['service_watch'] = $this->price_watch->service_highlights();
        $data['product_slide'] = $this->price_watch->getSlideAveragePrices();
        $data['db'] = $this->db;


        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->groupBy('product_category');
        $record = $builder->get()->getResult();

        $products = [];
        foreach ($record as $row) {
            $min = $this->db->table('products');
            $min->selectAvg('product_price');
            $min->where(['product_category' => $row->product_category]);
            $minResult = $min->get()->getRow();
            $products[] = array(
                'day'   => $row->category_name,
                'sell' => floatval($minResult->product_price)
            );
        }
        $data['products'] = ($products);
        return view('web/price_watch', $data);
    }

    // Product market highlights
    public function product_highlights($id = null)
    {

        $query = $this->db->query("SELECT * FROM districts WHERE district_slug='$id'");
        $result = $query->getRow();
        $data['page_title'] = $result->district_name;
        $data['dis_id'] = $result->district_id;

        $data['product_watch'] = $this->price_watch->market_highlights_district($result->district_id);
        $data['product_slide'] = $this->price_watch->getDistrictProdSlideAvegs($result->district_id);
        $data['db'] = $this->db;


        $builder = $this->db->table('products');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_district' => $result->district_id]);
        $builder->groupBy('product_category');
        $record = $builder->get()->getResult();

        $district = [];
        foreach ($record as $row) {
            $min = $this->db->table('products');
            $min->selectAvg('product_price');
            $min->where(['product_category' => $row->product_category]);
            $minResult = $min->get()->getRow();
            $district[] = array(
                'day'   => $row->category_name,
                'sell' => floatval($minResult->product_price),
            );
        }
        $data['district'] = ($district);
        return view('web/district_product_watch', $data);
    }

    // Service market highlights
    public function service_highlights($id = null)
    {

        $query = $this->db->query("SELECT * FROM districts WHERE district_slug='$id'");
        $result = $query->getRow();
        $data['page_title'] = $result->district_name;
        $data['dis_id'] = $result->district_id;

        $data['service_watch'] = $this->price_watch->service_district_highlights($result->district_id);
        $data['product_slide'] = $this->price_watch->getDistrictServiceSlideAvegs($result->district_id);
        $data['db'] = $this->db;


        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->where(['service_district' => $result->district_id, 'service_status' => 1]);
        $builder->groupBy('service_category');
        $record = $builder->get()->getResult();

        $district = [];
        foreach ($record as $row) {
            $min = $this->db->table('services');
            $min->selectAvg('service_price');
            $min->where(['service_category' => $row->service_category]);
            $minResult = $min->get()->getRow();
            $district[] = array(
                'day'   => $row->service_type_name,
                'sell' => floatval($minResult->service_price),
            );
        }
        $data['district'] = ($district);
        return view('web/district_service_watch', $data);
    }

    // Market highlights
    public function market_highlights($id)
    {

        $query = $this->db->query("SELECT * FROM markets WHERE market_slug='$id'");
        $result = $query->getRow();
        $data['page_title'] = $result->market_name;
        $data['mkt_id'] = $result->market_id;

        $data['product_watch'] = $this->price_watch->market_details_highlights($result->market_id);
        $data['category_slide'] = $this->price_watch->market_details_highlights($result->market_id);
        $data['db'] = $this->db;

        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_market' => $result->market_id, 'product_status' => 1]);
        $builder->groupBy('product_category');
        $record = $builder->get()->getResult();

        $market = [];
        foreach ($record as $row) {
            $min = $this->db->table('products');
            $min->selectAvg('product_price');
            $min->where(['product_category' => $row->product_category]);
            $minResult = $min->get()->getRow();
            $market[] = array(
                'day'   => $row->category_name,
                'sell' => floatval($minResult->product_price),
            );
        }
        $data['market'] = ($market);

        return view('web/market_product_watch', $data);
    }
}
