<?php

namespace App\Models\Admin;

use CodeIgniter\Model;

class ProductsModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'products';
    protected $primaryKey       = 'product_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['product_status'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    // Fetch Products
    public function getProdutcs()
    {
        $builder = $this->db->table('products');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->orderBy('product_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get product details
    public function productDetails($id)
    {
        $builder = $this->db->table('products');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->where(['product_id' => $id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Count ratings
    public function countRatings($id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['reviewed_product' => $id]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Fetch Product Orders
    public function getProductOrders()
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id', 'users.id=orders.vendor_id');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Completed Orders
    public function getCompletedOrders()
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id', 'users.id=orders.vendor_id');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['order_status' => 1]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Pending Transactions
    public function getPendingOrders()
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id', 'users.id=orders.vendor_id');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['order_status' => 2]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Cancelled Transactions
    public function getCancelledOrders()
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id', 'users.id=orders.vendor_id');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['order_status' => 0]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Product Reviews
    public function product_reviews($id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->join('users', 'users.id=product_reviews.reviewer_id');
        $builder->join('products', 'products.product_id=product_reviews.reviewed_product');
        $builder->where(['reviewed_product' => $id]);
        $builder->orderBy('review_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Order Invoice
    public function getOrderInvoice($order_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['order_id' => $order_id]);
        $result = $builder->get()->getRow();
        return $result;
    }
}
