<?php

namespace App\Models\Admin;

use CodeIgniter\Model;

class ServicesModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'services';
    protected $primaryKey       = 'service_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['service_status'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    // Fetch Attributes
    public function getServices()
    {
        $builder = $this->db->table('services');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->orderBy('service_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get service details
    public function serviceDetails($id)
    {
        $builder = $this->db->table('services');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->where(['service_id' => $id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Count ratings
    public function countRatings($id)
    {
        $builder = $this->db->table('service_reviews');
        $builder->where(['reviewed_service' => $id]);
        $result = $builder->countAllResults();
        return $result;
    }

    // Product Reviews
    public function service_reviews($id)
    {
        $builder = $this->db->table('service_reviews');
        $builder->join('users', 'users.id=service_reviews.reviewer_id');
        $builder->join('services', 'services.service_id=service_reviews.reviewed_service');
        $builder->where(['reviewed_service' => $id]);
        $builder->orderBy('review_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get Attribute ID
    public function getAttributeID($id)
    {
        $builder = $this->db->table('attributes');
        $builder->where(['attribute_id' => $id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Fetch Product Orders
    public function getServicesOrders()
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Completed Orders
    public function getCompletedOrders()
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_order_status' => 1]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Pending Transactions
    public function getPendingOrders()
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_order_status' => 2]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Cancelled Transactions
    public function getCancelledOrders()
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_order_status' => 0]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get order invoice
    public function getServiceOrderInvoice($order_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_order_id' => $order_id]);
        $result = $builder->get()->getRow();
        return $result;
    }
}
