<?php

namespace App\Models\Api\Web;

use CodeIgniter\Model;

class SellersModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'users';
    protected $primaryKey       = 'id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = [];

    // Home Services
    public function listHomeSellers()
    {
        $builder = $this->db->table('users');
        $builder->select('id, store_name, profile_image, address');
        $builder->where(['category' => 2, 'verification' => 1]);
        $builder->orderBy('id', 'DESC');
        $result = $builder->get(10)->getResult();
        return $result;
    }

    // Get Seller Profile
    public function sellerProfile($id)
    {
        $builder = $this->db->table('users');
        $builder->select('id,firstname,lastname,slug,dob,phone,email,address,store_name,store_tagline,store_description,profile_image,active,verification, districts.district_name as district, markets.market_name as market');
        $builder->join('districts', 'districts.district_id=users.district');
        $builder->join('markets', 'markets.market_id=users.market');
        $builder->where(['category' => 2, 'verification' => 1, 'id' => $id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get Seller Products
    public function getSellerProducts($id)
    {
        $builder = $this->db->table('products');
        $builder->select('products.*, categories.category_name as category, users.store_name as seller, districts.district_name as district, markets.market_name as market');
        $builder->join('users', 'users.id=products.product_vendor');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('districts', 'districts.district_id=products.product_district');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->where('product_vendor', $id);
        $builder->orderBy('product_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get seller orders
    public function getProductOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->select('orders.*, users.firstname as firstname, users.lastname as lastname, units.unit_shortname as measurement, products.product_name as product');
        $builder->join('users', 'users.id=orders.user_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['vendor_id' => $user_id]);
        $builder->orderBy('order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get seller services
    public function getSellerServices($user_id)
    {
        $builder = $this->db->table('services');
        $builder->select('services.*, service_types.service_type_name as category, service_units.service_unit_name as offered, districts.district_name as district');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->join('districts', 'districts.district_id=services.service_district');
        $builder->where(['service_vendor' => $user_id]);
        $builder->orderBy('service_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get seller service orders
    public function getServiceOrders($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->select('service_orders.*, users.firstname as firstname, users.lastname as lastname, service_units.service_unit_name as offered, services.service_name as service');
        $builder->join('users', 'users.id=service_orders.s_user_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_vendor_id' => $user_id]);
        $builder->orderBy('s_order_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get product requests
    public function getRequests($user_id)
    {
        $builder = $this->db->table('product_requests');
        $builder->select('product_requests.*, users.firstname as firstname, users.lastname as lastname');
        $builder->join('users', 'users.id=product_requests.user_id');
        $builder->where(['vendor_id' => $user_id]);
        $builder->orderBy('request_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Product Reviews
    public function getProductReviews($user_id, $id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->select('product_reviews.*, users.firstname as firstname, users.lastname as lastname, users.profile_image as user_image, products.product_name as product');
        $builder->join('users', 'users.id=product_reviews.reviewer_id');
        $builder->join('products', 'products.product_id=product_reviews.reviewed_product');
        $builder->where(['r_product_vendor' => $user_id, 'reviewed_product' => $id]);
        $builder->orderBy('review_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Service Reviews
    public function getServiceReviews($user_id, $id)
    {
        $builder = $this->db->table('service_reviews');
        $builder->select('service_reviews.*, users.firstname as firstname, users.lastname as lastname, users.profile_image as user_image, services.service_name as service');
        $builder->join('users', 'users.id=service_reviews.reviewer_id');
        $builder->join('services', 'services.service_id=service_reviews.reviewed_service');
        $builder->where(['r_service_vendor' => $user_id, 'reviewed_service' => $id]);
        $builder->orderBy('review_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Fetch Categories
    public function getCategories(){
        $builder = $this->db->table('categories');
        $builder->orderBy('category_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // fetch units
    public function getUnits(){
        $builder = $this->db->table('units');
        $builder->orderBy('unit_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Fetch Types
    public function getServiceTypes()
    {
        $builder = $this->db->table('service_types');
        $builder->orderBy('service_type_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get units
    public function getServiceUnits()
    {
        $builder = $this->db->table('service_units');
        $builder->orderBy('service_unit_id', 'ASC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get districts
    public function getDistricts()
    {
        $builder = $this->db->table('districts');
        $builder->orderBy('district_name', 'ASC');
        $result = $builder->get()->getResult();
        return $result;
    }
}
