<?php

namespace App\Models\Forum;

use CodeIgniter\Model;

class Topics extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'forum_topics';
    protected $primaryKey       = 'topic_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['topic_group_id', 'topic_title', 'topic_slug', 'topic_description', 'topic_author'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'topic_created_at';
    protected $updatedField  = 'topic_updated_at';

    // Get all Topics
    public function getAllTopics()
    {
        $builder = $this->db->table('forum_topics');
        $builder->join('users', 'users.id=forum_topics.topic_author');
        $builder->join('farm_groups', 'farm_groups.group_id=forum_topics.topic_group_id');
        $builder->orderBy('topic_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Fetch topic discussions
    public function getTopicDiscussions($id)
    {
        $builder = $this->db->table('forum_discussions');
        $builder->join('users', 'users.id=forum_discussions.farmer_id');
        $builder->join('forum_topics', 'forum_topics.topic_id=forum_discussions.discussion_topic_id');
        $builder->where(['discussion_topic_id' => $id]);
        $builder->orderBy('discussion_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }
}
