<?php

namespace App\Models\Vendor;

use CodeIgniter\Model;

class ServicesModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'services';
    protected $primaryKey       = 'service_id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['service_ref', 'service_category', 'service_vendor', 'service_name', 'service_slug', 'service_keywords', 'service_meta_desc', 'service_desc', 'service_image', 'sku', 'service_unit', 'service_price', 'service_discount', 'service_district', 'service_status'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    // Get Services
    public function getServices($user_id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_vendor' => $user_id]);
        $builder->orderBy('service_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Banned Services
    public function getBannedServices($user_id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_status' => 0, 'service_vendor' => $user_id]);
        $builder->orderBy('service_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Fetch service by id
    public function getServiceID($id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_id' => $id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get recent products
    public function getRecentServices($user_id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->where(['service_vendor' => $user_id]);
        $builder->orderBy('service_id', 'DESC');
        return $builder->get(5)->getResult();
    }
}
