<?php

namespace App\Models\Web;

use CodeIgniter\Model;

class PriceWatchModel extends Model
{
    protected $DBGroup          = 'default';
    
    // Product highlights
    public function market_highlights()
    {

        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->join('districts', 'districts.district_id=products.product_district');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['sale_type' => 1]);
        $builder->groupBy('product_category');
        // $builder->distinct('product_market');
        return $builder->get(15)->getResult();
    }

    // Product highlights by District
    public function market_highlights_district($id)
    {

        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->join('districts', 'districts.district_id=products.product_district');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['sale_type' => 1, 'product_district' => $id]);
        $builder->groupBy('product_category');
        return $builder->get()->getResult();
    }

    // Service Highlights
    public function service_highlights()
    {

        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('districts', 'districts.district_id=services.service_district');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->groupBy('service_district');
        $builder->orderBy('service_price', 'DESC');
        return $builder->get(15)->getResult();
    }

    // Service District Highlights
    public function service_district_highlights($id)
    {

        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('districts', 'districts.district_id=services.service_district');
        $builder->where(['service_status' => 1, 'service_district' => $id]);
        $builder->groupBy('service_category');
        return $builder->get()->getResult();
    }

    // Market highlights
    public function market_details_highlights($id){
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->where(['product_status' => 1, 'product_market' => $id]);
        $builder->groupBy('product_category');
        return $builder->get()->getResult();
    }


    // Get average prices
    public function getSlideAveragePrices()
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('districts', 'districts.district_id=products.product_district');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->groupBy('product_category');
        return $builder->get()->getResult();
    }

    // District slide averages
    public function getDistrictProdSlideAvegs($id)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('districts', 'districts.district_id=products.product_district');
        $builder->join('markets', 'markets.market_id=products.product_market');
        $builder->where(['product_district' => $id, 'product_status' => 1]);
        $builder->groupBy('product_category');
        return $builder->get()->getResult();
    }

    // Service slide averages
    public function getDistrictServiceSlideAvegs($id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('districts', 'districts.district_id=services.service_district');
        $builder->where(['service_district' => $id, 'service_status' => 1]);
        $builder->groupBy('service_category');
        return $builder->get()->getResult();
    }
    
}
