<?php

namespace App\Models\Web;

use CodeIgniter\Model;

class ServicesModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'services';
    protected $primaryKey       = 'id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = [];

    // Fetch service categories
    public function getPopularCategories()
    {

        $builder = $this->db->table('service_types');
        $builder->orderBy('service_type_id', 'RANDOM');
        $result = $builder->get(12)->getResult();
        return $result;
    }
    // Fetch all service types
    public function getAllServiceCategories()
    {

        $builder = $this->db->table('service_types');
        $builder->orderBy('service_type_id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Get popular districts
    public function getRandomDistricts()
    {
        $builder = $this->db->table('districts');
        $builder->orderBy('district_id', 'RANDOM');
        $result = $builder->get(20)->getResult();
        return $result;
    }

    // Fetch district id
    public function getFilterDistricts($slug)
    {
        $builder = $this->db->table('districts');
        $builder->where('district_slug', $slug);
        $result = $builder->get()->getRow();
        return $result;
    }
    // Fetch services by district
    public function service_by_district($slug)
    {
        $builder = $this->db->table('services');
        $builder->where('district_slug', $slug);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get product details
    public function getServiceDetails($slug)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_slug' => $slug]);
        return $builder->get()->getRow();
    }

    // Get product reviews
    public function getServiceReviews($id)
    {
        $builder = $this->db->table('service_reviews');
        $builder->join('users', 'users.id=service_reviews.reviewer_id');
        $builder->where(['reviewed_service' => $id]);
        return $builder->get()->getResult();
    }

    // Fetch Related products
    public function getRelatedServices($cat_id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_category' => $cat_id]);
        return $builder->get(4)->getResult();
    }

    // Get services by category
    public function getServicesByCategory($cat_id)
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_category' => $cat_id]);
        return $builder->get(4)->getResult();
    }

    // Count service by category
    public function countServiceByCategory($cat_id)
    {
        $builder = $this->db->table('services');
        $builder->where(['service_category' => $cat_id]);
        return $builder->countAllResults();
    }

    // Count Product Reviews
    public function countReviews($service_id)
    {
        $builder = $this->db->table('service_reviews');
        $builder->where(['reviewed_service' => $service_id]);
        return $builder->countAllResults();
    }

    // Count service results
    public function countServiceResults($district_id)
    {
        $builder = $this->db->table('users');
        $builder->where(['district' => $district_id, 'category' => 2, 'verification' => 1]);
        return $builder->countAllResults();
    }

    // Fetch recent services
    public function getRecentServices()
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->orderBy('service_id', 'DESC');
        return $builder->get(4)->getResult();
    }

    // Fetch all services
    public function getAllServices()
    {
        $builder = $this->db->table('services');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->orderBy('service_id', 'DESC');
        return $builder->get()->getResult();
    }
}
