<?php

namespace App\Models\Web;

use CodeIgniter\Model;

class UserModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'users';
    protected $primaryKey       = 'id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = ['billing_names', 'billing_phone', 'billing_address', 'billing_district', 'billing_city', 'profile_image', 'password_hash'];

    // Dates
    protected $useTimestamps = true;
    protected $dateFormat    = 'datetime';
    protected $createdField  = 'created_at';
    protected $updatedField  = 'updated_at';

    // Orders
    public function getOrders($user_id)
    {
        $builder = $this->db->table('orders');
        $builder->where(['user_id' => $user_id]);
        $builder->orderBy('order_id', 'DESC');
        return $builder->get()->getResult();
    }

    public function getServices($user_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->where(['s_user_id' => $user_id]);
        $builder->orderBy('s_order_id', 'DESC');
        return $builder->get()->getResult();
    }

    // Successful orders
    public function getSuccessfullOrders($user_id)
    {
        $builder1 = $this->db->table('orders');
        $builder1->where(['user_id' => $user_id, 'order_status' => 1]);

        $builder2 = $this->db->table('service_orders');
        $builder2->where(['s_user_id' => $user_id, 's_order_status' => 1]);

        return $builder1->countAllResults() + $builder2->countAllResults();
    }

    // Pending orders
    public function getPendingOrders($user_id)
    {
        $builder1 = $this->db->table('orders');
        $builder1->where(['user_id' => $user_id, 'order_status' => 2]);

        $builder2 = $this->db->table('service_orders');
        $builder2->where(['s_user_id' => $user_id, 's_order_status' => 2]);

        return $builder1->countAllResults() + $builder2->countAllResults();
    }

    // Cancelled orders
    public function getCancelledOrders($user_id)
    {
        $builder1 = $this->db->table('orders');
        $builder1->where(['user_id' => $user_id, 'order_status' => 0]);

        $builder2 = $this->db->table('service_orders');
        $builder2->where(['s_user_id' => $user_id, 's_order_status' => 0]);

        return $builder1->countAllResults() + $builder2->countAllResults();
    }

    // Get order invoice
    public function getOrderInvoice($invoice_id)
    {
        $builder = $this->db->table('orders');
        $builder->join('users', 'users.id=orders.vendor_id');
        $builder->join('units', 'units.unit_id=orders.pro_order_unit');
        $builder->join('products', 'products.product_id=orders.prod_id');
        $builder->where(['order_invoice' => $invoice_id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get service invoice
    public function getServiceInvoice($invoice_id)
    {
        $builder = $this->db->table('service_orders');
        $builder->join('users', 'users.id=service_orders.s_vendor_id');
        $builder->join('service_units', 'service_units.service_unit_id=service_orders.s_service_unit');
        $builder->join('services', 'services.service_id=service_orders.s_service_id');
        $builder->where(['s_order_invoice' => $invoice_id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get User Settings
    public function getUserSettings($user_id)
    {
        $builder = $this->db->table('users');
        $builder->join('districts', 'districts.district_id=users.district');
        $builder->where(['id' => $user_id]);
        $result = $builder->get()->getRow();
        return $result;
    }

    // Get districts
    public function getDistricts()
    {
        $builder = $this->db->table('districts');
        $builder->orderBy('district_name', 'ASC');
        $result = $builder->get()->getResult();
        return $result;
    }
}
