<?php

namespace App\Models\Web;

use CodeIgniter\Model;

class VendorsModel extends Model
{
    protected $DBGroup          = 'default';
    protected $table            = 'users';
    protected $primaryKey       = 'id';
    protected $useAutoIncrement = true;
    protected $insertID         = 0;
    protected $returnType       = 'array';
    protected $useSoftDeletes   = false;
    protected $protectFields    = true;
    protected $allowedFields    = [];

    // Fetch Vendors
    public function getVendors()
    {

        $builder = $this->db->table('users');
        $builder->join('districts', 'districts.district_id=users.district');
        $builder->where(['category' => 2, 'verification' => 1]);
        $builder->orderBy('id', 'DESC');
        $result = $builder->get(16)->getResult();
        return $result;
    }

    // Fetch Vendors by District
    public function getVendorsByDistrict($district_id)
    {

        $builder = $this->db->table('users');
        $builder->join('districts', 'districts.district_id=users.district');
        $builder->where(['category' => 2, 'verification' => 1, 'district' => $district_id]);
        $builder->orderBy('id', 'DESC');
        $result = $builder->get()->getResult();
        return $result;
    }

    // Fetch vendor by id
    public function getVendorProfile($slug)
    {
        $builder = $this->db->table('users');
        $builder->join('districts', 'districts.district_id=users.district');
        $builder->join('markets', 'markets.market_id=users.market');
        $builder->where(['category' => 2, 'verification' => 1, 'slug' => $slug]);
        $result = $builder->get()->getRow();
        if ($result) {
            return $result;
        } else {
            return false;
        }
    }

    // Fetch vendor products
    public function getVendorProducts($vendor_id)
    {
        $builder = $this->db->table('products');
        $builder->join('categories', 'categories.category_id=products.product_category');
        $builder->join('units', 'units.unit_id=products.product_unit');
        $builder->where(['product_vendor' => $vendor_id, 'product_status' => 1]);
        return $builder->get()->getResult();
    }

    // Fetch vendor services
    public function getVendorServices($vendor_id)
    {
        $builder = $this->db->table('services');
        $builder->join('users', 'users.id=services.service_vendor');
        $builder->join('service_types', 'service_types.service_type_id=services.service_category');
        $builder->join('service_units', 'service_units.service_unit_id=services.service_unit');
        $builder->where(['service_vendor' => $vendor_id, 'service_status' => 1]);
        return $builder->get()->getResult();
    }

    // Get featured vendors
    public function getFeaturedVendors()
    {
        $builder = $this->db->table('service_orders');
        // $builder->select('(SELECT SUM(service_orders.s_order_total) FROM service_orders) AS total_amount', false);
        $builder->distinct();
        $builder->join('users', 'users.id=service_orders.s_vendor_id');
        $builder->orderBy('s_order_total', 'DESC');
        return $builder->get()->getResult();
    }
    public function countVendors($id)
    {
        $builder = $this->db->table('users');
        $builder->where(['district' => $id]);
        return $builder->countAllResults();
    }

    // Count vendor products
    public function countProducts($user_id)
    {
        $builder = $this->db->table('products');
        $builder->where(['product_vendor' => $user_id]);
        return $builder->countAllResults();
    }

    // Count vendor services
    public function countServices($user_id)
    {
        $builder = $this->db->table('services');
        $builder->where(['service_vendor' => $user_id]);
        return $builder->countAllResults();
    }

    // Count Vendor Requests
    public function countRequests($user_id)
    {
        $builder = $this->db->table('product_requests');
        $builder->where(['vendor_id' => $user_id]);
        return $builder->countAllResults();
    }

    // Count Vendor Reviews
    public function countReviews($user_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['r_product_vendor' => $user_id]);
        return $builder->countAllResults();
    }

    // Count star rating
    public function _5Star($user_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['review_rating' => 5, 'r_product_vendor' => $user_id]);
        return $builder->countAllResults();
    }
    public function _4Star($user_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['review_rating' => 4, 'r_product_vendor' => $user_id]);
        return $builder->countAllResults();
    }
    public function _3Star($user_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['review_rating' => 3, 'r_product_vendor' => $user_id]);
        return $builder->countAllResults();
    }
    public function _2Star($user_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['review_rating' => 2, 'r_product_vendor' => $user_id]);
        return $builder->countAllResults();
    }
    public function _1Star($user_id)
    {
        $builder = $this->db->table('product_reviews');
        $builder->where(['review_rating' => 1, 'r_product_vendor' => $user_id]);
        return $builder->countAllResults();
    }

    // Fetch Similar Vendors
    public function getSimilarVendors($market_id)
    {
        $builder = $this->db->table('users');
        $builder->where(['category' => 2, 'verification' => 1, 'market' => $market_id]);
        $builder->orderBy('id', 'DESC');
        $result = $builder->get(4)->getResult();
        return $result;
    }

    // Fetch Recent Vendors
    public function getRecentVendors()
    {
        $builder = $this->db->table('users');
        $builder->where(['category' => 2, 'verification' => 1]);
        $builder->orderBy('id', 'DESC');
        $result = $builder->get(6)->getResult();
        return $result;
    }
}
