<?= $this->extend('vendors/layouts/app.php'); ?>

<?= $this->section('page_title'); ?>
Edit Services
<?= $this->endSection() ?>

<?= $this->section('body'); ?>
<div id="main-content">
    <div class="container-fluid">
        <div class="block-header">
            <div class="row clearfix">
                <div class="col-md-6 col-sm-12">
                    <h1>AMIS Uganda</h1>
                    <nav aria-label="breadcrumb">
                        <ol class="breadcrumb">
                            <li class="breadcrumb-item"><a href="#">Home</a></li>
                            <li class="breadcrumb-item"><a href="#">Page</a></li>
                            <li class="breadcrumb-item active" aria-current="page"><?= $breadcrumb; ?></li>
                        </ol>
                    </nav>
                </div>
                <div class="col-md-6 col-sm-12 text-right hidden-xs">
                    <a href="<?= site_url('seller/services') ?>" class="btn btn-sm btn-success"> Service List</a>
                </div>
            </div>
        </div>
        <div class="row clearfix">
            <div class="col-md-12">
                <div class="card">
                    <?php if (isset($validation)) : ?>
                        <div class="alert alert-danger" role="alert">
                            <?= $validation->listErrors() ?>
                        </div>
                    <?php endif; ?>

                    <?php if (session()->getTempdata('error')) : ?>
                        <div class="alert alert-danger alert-dismissible" role="alert">
                            <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                            <i class="fa fa-times-circle"></i> <?= session()->getTempdata('error'); ?>
                        </div>
                    <?php endif; ?>

                    <div class="body">
                        <form id="basic-form" method="post" action="<?= site_url('seller/edit_service/' . esc($service->service_id)) ?>" enctype="multipart/form-data" novalidate>
                            <?= csrf_field() ?>
                            <input type="text" name="vendor" class="form-control" value="<?= user()->id ?>" hidden>
                            <div class="form-group">
                                <label>Service Name <span class="text-danger">*</span></label>
                                <input type="text" name="name" class="form-control" value="<?= esc($service->service_name) ?>" required>
                            </div>
                            <div class="form-group">
                                <label>Service Slug <span class="text-success">(Auto generated)</span></label>
                                <input type="text" name="slug" class="form-control" value="<?= esc($service->service_slug) ?>" readonly>
                            </div>
                            <div class="form-group">
                                <label>Meta Keywords <span class="text-danger">*</span></label>
                                <input type="text" name="keywords" class="form-control" value="<?= esc($service->service_keywords) ?>" required>
                                <small class="text-success">To be used for SEO</small>
                            </div>
                            <div class="form-group">
                                <label>Meta Description <span class="text-danger">*</span></label>
                                <textarea name="meta_desc" class="form-control" rows="3" required><?= esc($service->service_meta_desc) ?></textarea>
                                <small class="text-success">To be used for SEO</small>
                            </div>
                            <div class="row">
                                <div class="col-lg-4">
                                    <div class="form-group">
                                        <label>Service Category <span class="text-danger">*</span></label>
                                        <select class="custom-select" name="category" required>
                                            <option value="<?= esc($service->service_category) ?>" selected><?= esc($service->service_type_name) ?></option>
                                            <?php foreach ($categories as $category) : ?>
                                                <?php if (esc($service->service_category) != $category->service_type_id) : ?>
                                                    <option value="<?= $category->service_type_id ?>"><?= $category->service_type_name ?></option>
                                                <?php endif; ?>
                                            <?php endforeach; ?>
                                        </select>
                                    </div>
                                </div>
                                <div class="col-lg-4">
                                    <div class="form-group">
                                        <label>Charge Type <span class="text-danger">*</span></label>
                                        <select class="custom-select" name="units" required>
                                            <option value="<?= esc($service->service_unit) ?>" selected><?= esc($service->service_unit_name) ?></option>
                                            <?php foreach ($service_units as $unit) : ?>
                                                <?php if (esc($service->service_unit) != $unit->service_unit_id) : ?>
                                                    <option value="<?= $unit->service_unit_id ?>"><?= $unit->service_unit_name ?></option>
                                                <?php endif; ?>
                                            <?php endforeach; ?>
                                        </select>
                                    </div>
                                </div>
                                <div class="col-lg-4">
                                    <div class="form-group">
                                        <label>Service Price <span class="text-danger">*</span></label>
                                        <input type="number" name="price" class="form-control" value="<?= esc($service->service_price) ?>" required>
                                    </div>
                                </div>
                                <div class="col-lg-12">
                                    <div class="form-group">
                                        <label>Service Description <span class="text-danger">*</span></label>
                                        <textarea name="description" id="ckeditor" required><?= esc($service->service_desc) ?></textarea>
                                    </div>
                                </div>
                                <div class="col-lg-12">
                                    <div class="form-group">
                                        <label>Service Image <span class="text-success">(Optional)</span></label>
                                        <input type="file" name="service_image" class="form-control-file" accept="image/x-png,image/gif,image/jpeg">
                                        <input type="text" name="old_image" class="form-control" value="<?= esc($service->service_image) ?>" hidden>
                                    </div>
                                    <img src="<?= base_url('public/uploads/services/' . esc($service->service_image)) ?>" alt="Service" class="img-fluid" width="150">
                                </div>
                            </div>
                            <br>
                            <button type="submit" class="btn btn-primary">Update Data</button>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>

<?= $this->endSection(); ?>